/*
 Copyright (C) 2017 Andrew Wuensche
 
 This file is part of DDLab (Discrete Dynamics Lab) <www.ddlab.org>
 
 DDLab is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with DDLab.  If not, see <http://www.gnu.org/licenses/>.
 */

/********** network graph and attractor metagraph *****************/
#include "edecfunc.h"

/***************  list of functions in this file ************/
/*char jump_graph_table(unsigned int att_types,
  sk_att *sk,char field_or_net);*/
float draw_circle_graph(float c_scale,float *m_node_size,
              signed int X1, signed int Y1,
              signed int X2, signed int Y2,
	      signed int wox, signed int woy,
              float node_factor,double w_angle);
float draw_2d_graph(float c_scale,float *m_node_size,
              signed int X1, signed int Y1,
              signed int X2, signed int Y2,
	      signed int wox, signed int woy, float node_factor);
float draw_3d_graph(float c_scale,float *m_node_size,
              signed int X1, signed int Y1,
              signed int X2, signed int Y2,
	      signed int wox, signed int woy, float node_factor); 
void unscramble_graph(signed int wox, signed int woy);
void shuffle_order1(unsigned int *shuffarray, unsigned int size);
void reorder_rank(unsigned int *reorder_graph);
char save_load_graph_layout(char save_load);
void show_table(char type, signed int X1, signed int Y1,
		           signed int X2, signed int Y2);
void intest_path(char what);
void prob_ant_prompts(char show_ant, char rand_ant);
void prob_ant(float c_scale,float *m_node_size,
               signed int wox, signed int woy,
                    signed int X1,signed int Y1,
		       signed int X2,signed int Y2);
void draw_jump_graph(float c_scale,float *m_node_size,
               signed int wox, signed int woy,
                    signed int X1,signed int Y1,
		       signed int X2,signed int Y2);
void jump_graph_vertex(signed int zw,float node_size,
              float c_scale, unsigned char thiscolor,
               signed int X1,signed int Y1,
			  signed int X2,signed int Y2);
/*void thickline(char selflink,signed int x1, signed int y1,
	       signed int x2, signed int y2, 
                       unsigned char thiscolor, 
                       double thisthick, double maxthick);*/
void manipulate_jump_graph(signed int wox, signed int woy,
                           signed int X1,signed int Y1,
		           signed int X2,signed int Y2,
                           float c_scale,float *m_node_size,
                           float rot_angle);
char block_or_slice(unsigned int poss,unsigned int prev);
void rotate_component_graph(char fragment, 
                            char clockwise,float rot_angle,
                            signed int c_x, signed int c_y);
void flip_component_graph(char fragment, char which,
			  signed int c_x, signed int c_y);
void prompt_and_component(char components, char blockset,char link_step,
       unsigned int most_recent_node, unsigned int prev_node);
void disconnect_node(char what,
	 unsigned long *node_outputs, unsigned long *node_inputs,
	 char components, unsigned int poss, unsigned int prev_node);
void node_pair_link(char what,
   char components, unsigned int poss, unsigned int prev_node);
void graph_in_out_components(char components,char link_step,
                               unsigned int poss);
void compute_link_angle(signed int x1, signed int y1,
                        signed int x2, signed int y2);


/*for jumpgraph*/
unsigned long **basinjumps,**b_j;
unsigned int node_no,*outside_inputs;
signed int suppress_u_links=0,u_threshold=0,isolate_unreachable=0;
unsigned char *basin_color;
float linkangle,abs_angle,
      w_cyclerad,arrow_size; /*to expand/contract links, and arrows*/
unsigned int m_size;
double m_space;
char f_or_n, /*field_or_net for jump-graph*/
     scale_m_nodes,scale_links,no_links=0,node_numbers=1,
     *linknet; /*to seperate components*/
/*new variables Dec 2001*/
/*m_basin_volume is now unsigned long, global variable, allocated in run*.c*/
unsigned long *histnodesize; /*to scale histogram nodes according attractor frequency*/
unsigned char *netnodesize, /*to scale network nodes according to k*/
     show_arrows,  /*toggle arrows*/
     quick_move=0; /*tog between updating net while dragging or at end*/
char graph_postscript=0,
     *filename_graph_PS, /*to save the network or jump_graph as a vector postscript*/
     graph_hex2d; /*to force hex or square 2d*/
unsigned int startBlock,endBlock;

/*********** compute and show jump-graph table, 28 July 2000 ***********/
char jump_graph_table(unsigned int att_types,sk_att *sk,
                              char field_or_net){
  signed int xbox,z,u,i,temp,bitflip,btarget,btype,attno,
               temp_xoffset,temp_yoffset,value,end_wait_text;
  unsigned int zw,zz,z1,itcount,n,*wnhood,
                 max_period,max_trans;/*for histogram*/
  char **att_check,*temppoint,*current_state,match_found=0,jump_datapause;
  unsigned long total_steps=0, steps_so_far=0;
  float jump_startrad;    

    use_preset_basin_pos=0;
    temp_xoffset=xoffset;
    temp_yoffset=yoffset;
    no_links=0;

    jump_startrad=startrad; /*to change scale of basins in jump-graph*/
    jump_datapause=datapause; /*no need for datapause for basins in jump-graph*/
    datapause=0;

    /*reset: suppress links from unreachable basins*/
    suppress_u_links=isolate_unreachable=u_threshold=0;
    /*f_or_n=='f' for jump-graph of basin of attraction field*/  
    /*f_or_n=='n' for graph of network*/ 
    f_or_n=field_or_net;
    get_prompt_box(3,4);
    draw_prompt_box(1);
    if (f_or_n=='f' || jump_layout_only){
       /*close the basin .ps file, otherwise done in main*/
       if (basin_postScript) close(fid_PS); 

       node_no=basintypeno; /*number of basins*/
       if (jump_layout_only==1){ 
	  f_or_n='n';/*change to "network graph", with basintypeno nodes*/
          no_links=1;
       }
    }
    else if (f_or_n=='a') node_no=att_types; /*attractor types in att histogram*/
    else if (f_or_n=='n') {
       node_no=lineL; /*size of network*/
       no_links=0; 
       if (direction=='f' && limitcheck==0){ /*forwards only, for stp*/
          no_links=1;           
          draw_prompt_box(1);
          sprintf(textb[0],
          "show links -L (caution for large networks), layout only -def:");
          outtext(textb[0]);
          input_string(1,0);
          if (is[0]=='L') no_links=0;
       }
       else{
          draw_prompt_box(1);
          sprintf(textb[0],
          "no links -N, show links -def:");
          outtext(textb[0]);
          input_string(1,0);
          if (is[0]=='N') no_links=1;
       }
    }

    draw_prompt_box(1);
    if (f_or_n=='n' && jump_layout_only==1) sprintf(textb[0],
           "computing nodes for layout of %d basins, quit-q, or wait...",node_no);
    else if (f_or_n=='f' || f_or_n=='a') sprintf(textb[0],
           "computing jump-graph, %d basins, quit-q, or wait...",node_no);    
    else if (f_or_n=='n') 
       sprintf(textb[0],
           "computing network-graph, %d nodes, quit-q, or wait...",node_no);
    outtext(textb[0]);
    end_wait_text=getx;
    /*wont outtext without this - ??*/
    while(event_ready()==0) break;
    
    if (f_or_n=='a'){ /*for a jumpgraph for the attractor histogram*/
      unsigned int zu;

       /*nodesize proportional to attractor type frequency, 0 to n-1*/
       if ((histnodesize=
                (unsigned long*)calloc(node_no+1,sizeof(unsigned long)))
				     ==NULL) outof_memory("histnodesize");
       memmove(histnodesize,sk->att_type_no,
	               (node_no+1)*sizeof(unsigned long));
       
       if ((m_att_period=
             (unsigned int*)calloc(node_no+1,sizeof(unsigned int)))
				     ==NULL) outof_memory("m_att_period");
       memmove(m_att_period,sk->att_type_period,
                      (node_no+1)*sizeof(unsigned int));

       max_period=max_trans=0;
       for (zu=0;zu<=node_no;zu++){
	 if (m_att_period[zu]>max_period) max_period=m_att_period[zu];
         if (sk->att_type_avtrans[zu]>max_trans) 
                               max_trans=sk->att_type_avtrans[zu]; 
       }
       jump_graph=1;
    }

    if (f_or_n=='f' || f_or_n=='a'){       
       char *state1m,*state2m;

       m_size=lineL*(value_range-1); 
       if (f_or_n=='f') m_space=pow(value_range,(double)lineL);
       else if (f_or_n=='a') m_space=sk->att_sample_no;
          
       /*memory for current state*/
       if ((current_state=(char *)calloc(bytesL,sizeof(char)))
			      ==NULL) outof_memory("current_state");
       /*memory for pseudo-nhood*/
       if ((wnhood=(unsigned int *)calloc(nhood,sizeof(unsigned int)))
			      ==NULL) outof_memory("wnhood");
     
       /*create array of one state in each basin*/
       if ((att_check=(char**)calloc(node_no+1,sizeof(char*)))
			      ==NULL) outof_memory("rcheck");
       for (z=0;z<=node_no;z++){
          if ((att_check[z]=(char *)calloc(bytesL,sizeof(char)))
			      ==NULL) outof_memory("att_check[]");
       } 
          
       for (z=1;z<=node_no;z++) 
                   memmove(att_check[z],m_att[z][1],bytesL);
	  
       /*keep count of basin jumps*/
       if ((basinjumps=
              (unsigned long**)calloc(node_no+1,sizeof(unsigned long*)))
			      ==NULL) outof_memory("basinjumps");
       for (z=1;z<=node_no;z++){ 
              if ((basinjumps[z]=
                  (unsigned long *)calloc(node_no+1,sizeof(unsigned long)))
			      ==NULL) outof_memory("basinjumps[z]");
       }
       /*backup of basin jumps*/
       if ((b_j=
              (unsigned long**)calloc(node_no+1,sizeof(unsigned long*)))
			      ==NULL) outof_memory("b_j");
       for (z=1;z<=node_no;z++){ 
              if ((b_j[z]=
                  (unsigned long *)calloc(node_no+1,sizeof(unsigned long)))
			      ==NULL) outof_memory("b_j[z]");
       }

       /*fprintf(stderr,"\n -----------------4 about to run -----------");*/
       if ((state1m=(char *)calloc(bytesL,sizeof(char)))==NULL) outof_memory("state1m");
       if ((state2m=(char *)calloc(bytesL,sizeof(char)))==NULL) outof_memory("state2m");

       
       /*count steps to monitor time*/
       for (btype=1;btype<=node_no;btype++){
	  for (attno=1;attno<=m_att_period[btype];attno++)    
	     total_steps+=1;
       }
       total_steps*=(lineL*value_range-1);

       for (btype=1;btype<=node_no;btype++){ 
          for (attno=1;attno<=m_att_period[btype];attno++){
             /*copy state*/
             memmove(current_state,m_att[btype][attno],bytesL);
             memmove(state1m,current_state,bytesL); /*copy state*/

             for (bitflip=L;bitflip>=0;bitflip--){ /*flip*/
	       for (value=0;value<=value_range-1;value++){
                      memmove(state1m,current_state,bytesL); /*restore state*/
                if ( get_value(state1m,bitflip)==value ) continue;
                else set_value(state1m,bitflip,value);
                steps_so_far++; 
                if (total_steps>100){              
                   if (steps_so_far%(total_steps/100)==0){
		      sprintf(textb[0],"%d%%",
                            (signed int)((steps_so_far*100)/total_steps));
		      inset_prompt_box(0,5,textb[0],'d');
                   }
		}
                itcount=0;
	        while(1){
		   if (event_ready()){
		      if (is[0]=='q'){
                         if (f_or_n=='a'){
			    free(histnodesize);
                            free(m_att_period);
                         }
                         if (f_or_n=='f' || f_or_n=='a'){                          
                            free(current_state);
                            free(wnhood);
                            for (z=1;z<=node_no;z++) free(att_check[z]);
                            free(att_check);
                            for (z=1;z<=node_no;z++) free(basinjumps[z]);
                            free(basinjumps);
                            free(b_j);
                            free(state1m);
	                    free(state2m);
                         }
                         return(0);
                      }
                   }                   
                   itcount++;
                   /*fprintf(stderr," %d",itcount);*/

                   if (exhaustive_testing) 
	               memmove(state2m,exh_pairs+dec_equiv(state1m,L)*bytesL,bytesL);
                   else{
                      for (n=L; n!=Maxu; n--){               
	                 /*local nhood with periodic boundary conditions*/
	                 if (nonlocal==0){
	                    /*i.e. for nhood=9 lineL=3 to work*/
	                    if (L<nhoodm[n]/2){
		               for (i=0;i<=nhoodm[n]-1;i++){
		                  temp=lineL+n-(nhoodm[n]/2)+i;
		                  if (temp<0) temp+=lineL;
		                  wnhood[i]=abs(temp)%lineL;
		               }
	                    }
	                    else{
		               for (i=0;i<=nhoodm[n]-1;i++)
			          wnhood[i]=(lineL+n-(nhoodm[n]/2)+i)%lineL;
	                    }
	                 }    
	                 /*set local 2d wiring*/
	                 else if (loc_twod_fonly){
                            if (twod==2) wire_2d_local(n,wnhood);
                            else if (threed==3) wire_3d_local(n,wnhood); 
                         }
	                 /*nonlocal=1, get psuedo-nhood from wiring scheme*/
	                 else{
	                    for (i=0;i<=nhoodm[n]-1;i++) 
                                      wnhood[i]=wiring[n][i];
	                 }

                         u=0; /*find dec value of actual or pseudo-nhood*/

                         /*xxxxxxxxxxxx next time-step xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx*/
                         if (tot_only==0)
                             u=next_time_step(carule,state1m,state2m,0,NULL,1,wnhood,n,0);
                          else{
			     if (totalistic==1) //run_forward(cacode);
                                u=next_time_step(cacode,state1m,state2m,0,NULL,1,wnhood,n,0); 
                             else if (totalistic==2) //run_forward(v_code);	      
	                        u=next_time_step(v_code,state1m,state2m,0,NULL,1,wnhood,n,0);        
			  }
                         /*xxxxxxxxxxxx end next time-step xxxxxxxxxxxxxxxxxxxxxxxxxxxxx*/



                         /*
                         for (i=nhoodm[n]-1;i>=0;i--){
	                    if (value_range==2) u+=get_bit(state1m,wnhood[i])*
                                   (signed int)pow(2,(double)i);
                             else u+=get_value(state1m,wnhood[i])*
                                   (signed int)pow(value_range,(double)i);
	                 }
	                 //set cell next line according to rule look-up table
                         if (!rulemix){
                            if (value_range==2){ 
                                 set_bit(state2m,n,get_bit(carule[0],u));
                            }
                            else set_value(state2m,n,get_value(carule[0],u));
	                 }
                         else{
		            if (value_range==2){
                               set_bit(state2m,n,get_bit(carule[n],u));
                            }
                            else set_value(state2m,n,get_value(carule[n],u));
			 }*/

     
	              }/*for loop*/
                   }
                   /*compare with attractor state*/
                   for (btarget=1;btarget<=node_no;btarget++){ 
		     if (memcmp(state2m,att_check[btarget],bytesL)==0){
		          basinjumps[btype][btarget]+=1;
                          b_j[btype][btarget]+=1;
                          /*fprintf(stderr,"\n match found\n ");*/
                          match_found=1;
                          break;			
                     }
                   }
                   if (f_or_n=='a'){
                     /*hist, if no match found in reasonable time*/
		     if (itcount>(max_period+max_trans)*3){
                          basinjumps[btype][0]+=1;
                          b_j[btype][0]+=1;
                          match_found=1;
                          break;			
                     }
                   }
                   if (match_found==1){
                      match_found=0;
                      break;
                   }                                   
                   temppoint=state1m; /*copy F to state1*/
	           state1m=state2m;
	           state2m=temppoint;                 
	        } /*while loop*/                                          	      
               }/*for (value=..*/
             }/*for (bitflip=..*/
          } /*for (attno=..*/ 
       }/*for (btype=..*/
       free(state1m);
       free(state2m);    
       free(current_state);
       free(wnhood);
       for (z=1;z<=node_no;z++) free(att_check[z]);
       free(att_check);
    }
     
    else if (f_or_n=='n'){ 
       unsigned long tot_wire=0;

       if (no_links==0){ 
        /*keep count of directed links in network, 0 to n-1*/      
        if ((basinjumps=
              (unsigned long**)calloc(node_no+1,sizeof(unsigned long*)))
			      ==NULL) outof_memory("basinjumps*");

        for (zw=1;zw<=node_no;zw++){ 
              if ((basinjumps[zw]=
                  (unsigned long *)calloc(node_no+1,sizeof(unsigned long)))
			      ==NULL) outof_memory("basinjumps[]");
        }
        if ((b_j=
              (unsigned long**)calloc(node_no+1,sizeof(unsigned long*)))
			      ==NULL) outof_memory("b_j*");

        for (zw=1;zw<=node_no;zw++){ 
              if ((b_j[zw]=
                  (unsigned long *)calloc(node_no+1,sizeof(unsigned long)))
			      ==NULL) outof_memory("b_j[]");
        }

        if ((netnodesize=
                (unsigned char*)calloc(node_no+1,sizeof(unsigned char)))
				     ==NULL) outof_memory("netnodesize");
        if ((m_att_period=
             (unsigned int*)calloc(node_no+1,sizeof(unsigned int)))
				     ==NULL) outof_memory("m_att_period"); 
       
        for (z1=1;z1<=node_no;z1++){
           for (zz=0;zz<=nhoodm[z1-1]-1;zz++){
             total_steps+=1;
           }
        }
        total_steps*=lineL;

        for (zw=1;zw<=node_no;zw++){
	  tot_wire+=nhoodm[zw-1];          
          for (z1=1;z1<=node_no;z1++){
	     for (zz=0;zz<=nhoodm[z1-1]-1;zz++){
               steps_so_far++;
               if (total_steps>100){
                  if (steps_so_far%(total_steps/100)==0){
		      sprintf(textb[0],"%d%%",
                            (signed int)((steps_so_far*100)/total_steps));
		      inset_prompt_box(0,5,textb[0],'d');
                  }
               } 
               if (event_ready()){ /*chance to give up for very big networks*/
		      if (is[0]=='q'){
                            for (z=1;z<=node_no;z++){
                               free(basinjumps[z]);
                               free(b_j[z]);
                            }
                            free(basinjumps);
                            free(b_j);
			    free(netnodesize);
                            free(m_att_period);
                         return(0);
                      }
               }  
	       if (wiring[z1-1][zz]==zw-1){
                  basinjumps[zw][z1]+=1;
                  b_j[zw][z1]+=1;
               }
	     }
	  }
        }
       
        for (zw=1;zw<=node_no;zw++){  
           netnodesize[zw]=nhoodm[zw-1];
           m_att_period[zw]=celloutfreq[zw-1];
        }
       }
       /*m_att_period .. celloutfreq*/
       /*m_lineL .. 1*/
       if (jump_layout_only==1){
          m_size=lineL*(value_range-1); 
          m_space=pow(value_range,(double)lineL);         
       }
       else{ 
          m_size=1;
          m_space=(double)tot_wire;
       }   
    }
          
    { /*draw jump-graph*/
           double w_angle;
           float rot_angle,node_factor,link_factor;
           float c_scale;
           double this_f,av_f;
           float *m_node_size;
           signed int X1,Y1,X2,Y2,wox,woy,promptx,prompty;
           char tog_window=0;
           char tog_rank=0;
           unsigned int *reorder_graph;
           signed int **temp_basin_pos;


           /*keep track of rank order*/
           if ((reorder_graph=(unsigned int*)
                      calloc(node_no+1,sizeof(unsigned int)))
					==NULL) outof_memory("reorder_graph");
           /*keep track of previous positions*/
           if ((temp_basin_pos=(signed int**)calloc(node_no+1,
                     sizeof(signed int*)))==NULL) outof_memory("temp_basin_pos");
                 for (zw=0;zw<=node_no;zw++)
                    if ((temp_basin_pos[zw]=(signed int *)
                         calloc(2,sizeof(signed int)))
			      ==NULL) outof_memory("basin_pos[]");
 
           /*create array of basin positions
                      [node_no][0]=x, [node_no][1]=y*/
           
           if ((basin_pos=(signed int**)calloc(node_no+1,
                  sizeof(signed int*)))==NULL) outof_memory("basin_pos");
           for (zw=0;zw<=node_no;zw++){
              if ((basin_pos[zw]=(signed int *)calloc(
                       2,sizeof(signed int)))
			      ==NULL) outof_memory("basin_pos[]");
           }
           /*array for scales node sizes*/          
           if ((m_node_size=(float *)calloc(node_no+1,sizeof(float)))
			      ==NULL) outof_memory("m_node_size");

           if ((basin_color=(unsigned char *)calloc(node_no+1,
                sizeof(unsigned char)))==NULL) outof_memory("basin_color");

           if ((outside_inputs=(unsigned int *)calloc(node_no+1,
                sizeof(unsigned int)))==NULL) outof_memory("outside_inputs");

           /*default 2d hex or square -- then 2d-2 toggles between hex/square*/
           if (f_or_n=='n' && twod && nhood_mixed==0 &&
                   (nhood==6 || nhood==7 || nhood==10 || nhood==15 ||
		    nhood==18 || nhood==19 || nhood==22) 
		          && i2d%2==0 && j2d%2==0){
		   graph_hex2d=1;
              }
	      else graph_hex2d=0;

           w_cyclerad=100+node_no/4;
           if (w_cyclerad>=133) w_cyclerad=133;
           w_cyclerad=vgax(w_cyclerad);

           /*whole box*/
           X1=vgax(5);
           X2=Maxx-vgax(5);
           Y1=CHeight*5; /*higher box for networks or att hist jump graph
			   otherwise leave space for b-of-a field*/
           if (f_or_n=='f' || jump_layout_only) Y1+=(CHeight*7);
           Y2=vgay(430);
           if (f_or_n=='a') Y2-=(CHeight*7); /*to keep att hist visible*/
           draw_box(3,X1,Y1,X2,Y2,1,0);

           wox=Maxx/2;
           woy=Y1+(Y2-Y1)/2;
           /*woy=bottom-w_cyclerad-vgax(10);*/
     
           w_angle=2*PI/node_no;
           if (no_links==1) scale_m_nodes=0;
           else scale_m_nodes=1;
           if (f_or_n=='n') scale_links=0; /*at start, dont scale the links for network-graph*/
           else scale_links=1;             /*scale the links for jump-graph only*/
           if (jump_layout_only) scale_m_nodes=1; /*scale nodes for layout-graph*/

           show_arrows=1;
           rot_angle=2*PI/24; /*default rotation angle=15 degrees*/
           node_factor=1.2; /*to expand/contract nodes*/
           link_factor=1.1; /*to expand/contract links, and arrows*/
           arrow_size=0.07; /*arrow size * w_cyclerad*/
           /*note total inputs from outside, 
             to check unreachable basins*/
           if (no_links==0){
              for (zw=1;zw<=node_no;zw++){
                 for (btarget=1;btarget<=node_no;btarget++){
	            if (btarget!=zw) 
		    outside_inputs[zw]+=basinjumps[btarget][zw];
	         }
              }
           }
           /* draw default graph */           
           if (oned==1) c_scale=draw_circle_graph(
                   c_scale,m_node_size,X1,Y1,X2,Y2,wox,woy,
                            node_factor,w_angle);
           else if (twod==2) c_scale=draw_2d_graph(
                   c_scale,m_node_size,X1,Y1,X2,Y2,wox,woy,
                            node_factor);
           else if (threed==3) c_scale=draw_3d_graph(
                   c_scale,m_node_size,X1,Y1,X2,Y2,wox,woy,
                            node_factor);
	  startprompt:
           graph_postscript=0;/*to save the network or jump_graph as a vector postscript*/ 
           /*save area under prompt box in buf 3, 4 lines*/
           setcolor(white);      
           xbox=draw_prompt_box(4);
           if (jump_layout_only) outtext(
             "JUMP-graph (no edges): drag-(def) PScript-P tog:win-w rank-k");
           else if (f_or_n=='f' || f_or_n=='a') outtext( 
            	/*"JUMP-graph: drag-(def) ant-a path-p unscram-u tog:win-w rank-k");*/
                "JUMP-graph: drag-(def) PScript-P ant-a unscram-u tog:win-w rank-k");
           else{ 
              if (no_links==0) outtext(
	       /*"NETWORK-graph: drag-(def) path-p ant-a unscram-u tog:win-w rank-k");*/
                 "NETWORK-graph: drag-(def) PScript-P ant-a unscram-u tog:win-w rank-k");
              else outtext(
                "NETWORK-graph (no links): drag-(def) PScript-P tog:win-w");
           } 
           moveto(xbox,gety+CHeight);
            outtext(
              "settings-S rotate-x/X flip-h/v exp/contr:nodes-e/c links-E/C both-B/b");
           moveto(xbox,gety+CHeight);
           if (jump_layout_only) outtext(
                "basins-i/I/s tog: nodes-n/N");
	   else if (f_or_n=='f') outtext(
              "basins-i/I/s Unreach-U tog:table-t/T nodes-n/N links-l arrows-A</>");
           else{ 
             if (no_links==0) outtext(             
                 "Unreach-U tog: table-t/T nodes-n/N links-l arrows-A/</>");
             else outtext(             
                       "nodes-n/N");
           }
           moveto(xbox,gety+CHeight);
           outtext(
             "layout:file-f circle/spiral-o/O 1d/2d(tog)/3d-1/2/3 rnd-r/R quit-q:");
           promptx=getx;
           prompty=gety;

           startbox:
           use_preset_basin_pos=0;
           moveto(promptx,prompty);
#          if __UNIX__
	      /*while(event_ready()==0);*/ /*255 to suppress flashing cursor*/
              flashing_cursor(0,0,0,0,0,-1,CWidth,1+CHeight/2,255);
#          elif __WATCOM__
              if (mouse_found) mouse_call(1,0,0,0); /*display mouse*/
              flashing_cursor(0,CHeight-vgay(3),CWidth,CHeight,green);
#          endif
           if (button=='l' || button=='r') is[0]=13;
           else{
               char test=0;
	        test=keycheck(0);
                is[0]=test;
           }

           /* jump-graph, layout-graph and network graph: on-the-fly options
              gGjJKLmMPqQRsTVWyYzZ - unused
           > expand arrows
           < contract arrows
           1 1d layout
           -----------
           2 2d layout
           2x override 2d hex layout
           2s override 2d square layout
           -----------
           3 3d layout
          13 (return) drag
           a launch probabilistic ant in jumpgraph
           A toggle arrows on links
           b contract both links and nodes 
           B expand both links and nodes 
           c contract nodes
           C contract links
           d redraw current graph
           e expand nodes
           E expand links
           f save/load the graph layout
           h flip accos horizontal axis
           i redraw the field with basins at node positions - without nodes
           I redraw the field with basins at node positions - inside nodes
           k rank and reposition in descending order or links or basin volume
           l toggle links: same thin or scaled
           n toggle nodes: same or scaled according to basin volume or k for networks
           N toggle node numbers in nodes
           o circle layout
           O spiral layout
 removed - p hortest path (or small world distance)
           P vector postScipt of graph
           r randomize the existing layout by a small amount
           R random layout
           S various settings 
           t show table - numbers
           T show table - fractions
           u unscramble graph automatically
           U suppress links from unreachable nodes/basins
           v flip accoss vertical axis
           w redraw current graph
           x rotate clockwise
           X rotate anti-clockwise
	   */

           /*any key to restore jump-graph if showing basins below jump-graph*/
           if (tog_window==1){ 
                 draw_box(-1,X1,Y1,X2,Y2,1,0);/*clear box*/
                 draw_jump_graph(c_scale,m_node_size,wox,woy,X1,Y1,X2,Y2);
                 tog_window=0;
                 goto startprompt;
           }
           is[1]=0;
           switch(is[0]){
            case 13: /*change to dragging nodes etc*/
                sprintf(textb[9],"%s","boximage_prompt[3]");
                putimage(trboxw,2,boximage_prompt[3],COPY_PUT,textb[9]);
                manipulate_jump_graph(wox,woy,X1,Y1,X2,Y2,
                                c_scale,m_node_size,rot_angle);
                is[0]=0;
                goto startprompt;
	    case 'P':{ /*save the graph as a vector postScript file*/	      
	        erase_prompt_box(4);
		draw_prompt_box(1);
                outtext("PostScript: greyscale-P color-p:");
                input_string(1,0);
                if (is[0]=='p'|| is[0]=='P'){
                  signed int temp_fid_PS;
                  char file_ret,temp_basin_postScript;
		   /*disable basin_postScript*/
		   temp_fid_PS=fid_PS; /*to restore fid_PS (global variable) afterwards*/

		   temp_basin_postScript=basin_postScript;
		   basin_postScript=0;
                   if (is[0]=='p') graph_postscript=1;
                   else if (is[0]=='P') graph_postscript=2;
                   /*filename for graph_postScript ascii file*/
                   if (f_or_n=='f') /*for jump-graph of basin of attraction field*/
                     file_ret=select_filename("SAVE","PostScript file","*.ps","my_jgPS"); 
                   else /*f_or_n=='n' for graph of network*/
                     file_ret=select_filename("SAVE","PostScript file","*.ps","my_ngPS");
	           if (file_ret=='q') goto startprompt;
                   if ((filename_graph_PS=(char *)calloc(13,sizeof(char)))
				   ==NULL) outof_memory("filename_graph_PS");
	           memmove(filename_graph_PS,filename,13);
                   
                   /*delete previous*/
                   fid_PS=myopen(filename_graph_PS,1,0);
                   vret=write(fid_PS, "",0);
                   close(fid_PS);
                   /*start new*/
                   fid_PS=myopen(filename_graph_PS,1,1);
                   if (fid_PS==-1){ 
                       file_error(filename_graph_PS);
                       goto skip_ps;
                   }                          
                   prepare_basinPS(2,1); /*write/define postScipt for graph*/
                   draw_jump_graph(c_scale,m_node_size,wox,woy,X1,Y1,X2,Y2);
		  skip_ps:
                   close(fid_PS); 
                   graph_postscript=0;
                   free(filename_graph_PS);
                   basin_postScript=temp_basin_postScript;
                   fid_PS=temp_fid_PS; /*to restore fid_PS (global variable) afterwards*/
                }
                goto startprompt;
	    }
            case 'f':{ /*save/load the graph layout*/
                char ok,save_load;
                 erase_prompt_box(4);
		 draw_prompt_box(1); 
                 outtext("graph layout file: save-s load-l:");
                 input_string(1,0);
                 if (is[0]=='s' || is[0]=='l'){
                   save_load=is[0];
	           ok=save_load_graph_layout(is[0]);		
                   if (save_load=='l' && ok==1){
                      draw_box(-1,X1,Y1,X2,Y2,1,0);/*clear box*/
                      draw_jump_graph(c_scale,m_node_size,wox,woy,X1,Y1,X2,Y2);
                   }
                   is[0]=0;
                 }
                 goto startprompt;
            }
            case 'd': /*redraw current graph*/	      
                draw_box(-1,X1,Y1,X2,Y2,1,0);/*clear box*/
                draw_jump_graph(c_scale,m_node_size,wox,woy,X1,Y1,X2,Y2);
                break;           
            case 'w': /*tog window to see basins*/
              /*if tog_window==1 any key will restore - see above switch(is[0])*/
	      if (tog_window==0){ 
                 sprintf(textb[9],"%s","boximage[3]");
                 putimage(X1,Y1,boximage[3],COPY_PUT,textb[9]);/*below the jump-box*/
		 if (f_or_n=='f'){
		   //putimage((signed int)((float)Maxx)/2,2,/*the field info box*/
                   putimage(trboxw,2,/*the field info box*/
				  boximage_prompt[3],COPY_PUT,textb[9]);
                    print_field_data();
                    inset_prompt_box(0,20,"restore jump-graph -any key",'d');
                 }
                 tog_window=1;
              }              
              break;
            case 'a': /*launch probabilistic ant in jumpgraph*/
              if (no_links==1)  goto startprompt;              
              erase_prompt_box(4);
              draw_box(-1,X1,Y1,X2,Y2,1,0);/*clear box*/
              draw_jump_graph(c_scale,m_node_size,wox,woy,X1,Y1,X2,Y2);
	      prob_ant(c_scale,m_node_size,wox,woy,X1,Y1,X2,Y2);
              is[0]=0;
              goto startprompt;

	    /*case 'p': 
              erase_prompt_box(4);               
              xbox=draw_prompt_box(2);
              outtext(
       "intest path or small world distance, between all pairs of nodes");
              moveto(xbox,gety+CHeight);
              outtext("directed intest path -p, non-directed distance-d:");
              input_string(1,0);
              if (is[0]=='p') intest_path('p');
              else if (is[0]=='d') intest_path('d');
              draw_box(-1,X1,Y1,X2,Y2,1,0);//clear box
              draw_jump_graph(c_scale,m_node_size,wox,woy,X1,Y1,X2,Y2);
              goto startprompt;*/

            case 't': case 'T':  /*show table - numbers - fractions*/
              if (no_links==1)  goto startprompt;
              erase_prompt_box(4);                   
              show_table(is[0],X1,Y1,X2,Y2);
              draw_box(-1,X1,Y1,X2,Y2,1,0);/*clear box*/
              draw_jump_graph(c_scale,m_node_size,wox,woy,X1,Y1,X2,Y2);
              is[0]=0;
              goto startprompt;
            case 'U': /*suppress links from unreachable basins*/
                 if (no_links==1)  goto startprompt;                 
                 erase_prompt_box(4);               
                 xbox=draw_prompt_box(3);
                 suppress_u_links=0;
                 outtext("links from unreachable nodes: suppress-u restore-(ret):");
                 input_string(1,0);
                 if (is[0]=='u') suppress_u_links=1;
                 if (suppress_u_links==1){ /*unreachable nodes*/
                    isolate_unreachable=0;
                    moveto(xbox,gety+CHeight);
                    outtext("isolate unreachable -i:");
                    input_string(1,0);
                    if (is[0]=='i') isolate_unreachable=1;
                    moveto(xbox,gety+CHeight);
                    sprintf(textb[0],
                      "change unreachable threshold (now %d):",u_threshold);
                    outtext(textb[0]);
                    input_string(3,0);
                    if (is[0]!=13 && atoi(is)>=0) u_threshold=atoi(is);

                    if (isolate_unreachable==1){
                       for (zw=node_no;zw>=1;zw--){
		          if (outside_inputs[zw]<=u_threshold){
                             /*position at random on left 1/5 window*/
		             basin_pos[zw][0]=rand()%((X2-X1-vgax(60))/5) 
                                                            + X1+vgax(30);
                          }
	               }
                    }
                 }
                 draw_box(-1,X1,Y1,X2,Y2,1,0);/*clear box*/
                 draw_jump_graph(c_scale,m_node_size,wox,woy,X1,Y1,X2,Y2);
                 goto startprompt;  
            case 'n': /*toggle nodes: same or scaled according to basin volume, 
                                           or k for networks*/
              if (no_links==1 && jump_layout_only==0)  goto startprompt;             
	      scale_m_nodes=abs(scale_m_nodes-1); 
              draw_box(-1,X1,Y1,X2,Y2,1,0);/*clear box*/
              draw_jump_graph(c_scale,m_node_size,wox,woy,X1,Y1,X2,Y2);
              break;

	    case 'N': /* toggle node numbers in nodes */
              draw_box(-1,X1,Y1,X2,Y2,1,0);/*clear box*/
              node_numbers=abs(node_numbers-1);
              draw_jump_graph(c_scale,m_node_size,wox,woy,X1,Y1,X2,Y2);
              break;
            case 'l': /*toggle links: same thin or scaled according to fraction outputs and
                                  node size (basin volume, or k for networks)*/
              if (no_links==1)  goto startprompt;
	      scale_links=abs(scale_links-1); 
              draw_box(-1,X1,Y1,X2,Y2,1,0);/*clear box*/
              draw_jump_graph(c_scale,m_node_size,wox,woy,X1,Y1,X2,Y2);
              break;
            case 'A': /*toggle arrows on links*/
              if (no_links==1)  goto startprompt;              
              show_arrows=abs(show_arrows-1);
              draw_box(-1,X1,Y1,X2,Y2,1,0);/*clear box*/
              draw_jump_graph(c_scale,m_node_size,wox,woy,X1,Y1,X2,Y2);
              break;
             case '<': case '>': /*expand/contract arrows*/
              if (no_links==1 || show_arrows==0)  goto startprompt;
              if (is[0]=='>') arrow_size*=1.1;
              else arrow_size*=0.9;
              draw_box(-1,X1,Y1,X2,Y2,1,0);/*clear box*/
              draw_jump_graph(c_scale,m_node_size,wox,woy,X1,Y1,X2,Y2);
              break;
            case 'o': /*circle layout*/
              c_scale=draw_circle_graph(
                 c_scale,m_node_size,X1,Y1,X2,Y2,wox,woy,node_factor,w_angle);
              break;
	    case 'O':{ /*spiral: set and remember initial basin positions*/
	      float start_spiral, start_angle,s_s,s_a;
              signed int bcount=0;
             
              /*adjust size, angle and nodes for spiral*/
              start_spiral=40 * (pow(100,0.3)/pow(node_no,0.3));
              s_s=vgax(start_spiral);
              start_angle=2*PI/(node_no/10);            
              s_a=start_angle;

              /*reset node scale*/
              c_scale=PI*w_cyclerad/(node_no*1.2);
              if (c_scale>w_cyclerad/5) c_scale=w_cyclerad/5; 
              c_scale*=(node_factor*(pow(node_no,0.7)/pow(100,0.7)));
              if (c_scale<2.5) c_scale=2.5;
              for (zw=node_no;zw>=1;zw--){
                 if (no_links==1 && jump_layout_only==0) m_node_size[zw]=c_scale;
                 else{
		    av_f= m_space / node_no; 
                    if (f_or_n=='f' || jump_layout_only){ 
                        this_f=(double)m_basin_volume[zw];
                    }
                    else if (f_or_n=='a')
                          this_f=(double)histnodesize[zw];
                    else if (f_or_n=='n') this_f=(double)netnodesize[zw];
                  
	            m_node_size[zw]=c_scale*(float)pow(this_f/av_f,.5);
                    /*fprintf(stderr,"\n m_node_size[%d]=%.1f ",zw,m_node_size[zw]);*/ 
       
                 }
		 bcount++;	     
                 basin_pos[zw][0]=(cos(-(s_a*zw))*s_s)+wox;
                 basin_pos[zw][1]=-(sin(-(s_a*zw))*s_s)+woy;
                 s_s*=(1+0.2/pow(bcount,0.9));
                 s_a/=(1+0.01/pow(bcount,0.7));
              }
              draw_box(-1,X1,Y1,X2,Y2,1,0);/*clear box*/
              draw_jump_graph(c_scale,m_node_size,wox,woy,X1,Y1,X2,Y2);
              break;           
            } 
            case 'k':{
	         /*rank and reposition in descending order or links or basin volume*/           
                 if (no_links==1 && jump_layout_only==0)  goto startprompt;
                 if (tog_rank==1){ /*tog to previous*/
                    for (zw=0;zw<=node_no;zw++)           
		      memmove(basin_pos[zw],
		          temp_basin_pos[reorder_graph[zw]],2*sizeof(signed int));
                 }
                 else if (tog_rank==0){ /*tog to rank order*/
                       for (zw=1;zw<=node_no;zw++) reorder_graph[zw]=zw;
                    reorder_rank(reorder_graph);

                    for (zw=node_no;zw>=1;zw--){     
                       temp_basin_pos[reorder_graph[zw]][0]=basin_pos[zw][0];
                       temp_basin_pos[reorder_graph[zw]][1]=basin_pos[zw][1];
                    }
                    for (zw=0;zw<=node_no;zw++)           
		           memmove(basin_pos[zw],temp_basin_pos[zw],2*sizeof(signed int));

                 }
                 draw_box(-1,X1,Y1,X2,Y2,1,0);/*clear box*/
                 draw_jump_graph(c_scale,m_node_size,wox,woy,X1,Y1,X2,Y2);
                 tog_rank=abs(tog_rank-1);                                      
                 break;
           }

	   case 'u': /*unscramble graph automatically*/
              if (no_links==1)  goto startprompt;
	      unscramble_graph(wox,woy);
              draw_box(-1,X1,Y1,X2,Y2,1,0);/*clear box*/
              draw_jump_graph(c_scale,m_node_size,wox,woy,X1,Y1,X2,Y2);
              is[0]=0;
              break;
	   case 'x': case 'X': /*rotate clockwise anti-clockwise*/
              rotate_component_graph(0,is[0],rot_angle,wox,woy);	     
              draw_box(-1,X1,Y1,X2,Y2,1,0);//clear box
              draw_jump_graph(c_scale,m_node_size,wox,woy,X1,Y1,X2,Y2);
              break;
	   case 'v': case 'h':{ /*flip accoss vertical or horizontal axis*/             
              flip_component_graph(0,is[0],wox,woy);
              draw_box(-1,X1,Y1,X2,Y2,1,0);//clear box
              draw_jump_graph(c_scale,m_node_size,wox,woy,X1,Y1,X2,Y2);
              break;
	   }
	   case 'S':{ /*various settings*/
             signed int this_angle;             
              xbox=draw_prompt_box(4);
              sprintf(textb[0],
              "settings: rotation angle (start=15), now %.2f degrees:", 
                                                 (rot_angle*57.2958));
              outtext(textb[0]);
              input_string(4,1);
              if (is[0]!=13){
                 this_angle=atoi(is);
                 rot_angle=2*PI*(float)this_angle/360;
              } 
              moveto(xbox,gety+CHeight);
              sprintf(textb[0],
                 "expand/contract nodes: factor (1.0 to 2.0, start=1.2), now %.2f:", 
                                                                 node_factor);
              outtext(textb[0]);
              input_string(4,1);
              if (is[0]!=13 && atof(is)<1 && atof(is)<=2) node_factor=atof(is);
              moveto(xbox,gety+CHeight);
              sprintf(textb[0],
                 "expand/contract links: factor (1.0 to 2.0, start=1.1), now %.2f:", 
                                                                 link_factor);
              outtext(textb[0]);
              input_string(4,1);
              if (is[0]!=13 && atof(is)<1 && atof(is)<=2) node_factor=atof(is);
              moveto(xbox,gety+CHeight);
              sprintf(textb[0],
                 "change arrow size: (0 to .3, start=0.07), now %.2f:",arrow_size);
              outtext(textb[0]);
              input_string(4,1);
              if (is[0]!=13 && atof(is)>=0 && atof(is)<=0.3) arrow_size=atof(is); 
              is[0]=0;
              goto startprompt;
           }
	   case 'e': case 'c': /*expand - contract nodes*/
	      if (is[0]=='e') c_scale*=node_factor;
              else if (is[0]=='c'){ 
                  c_scale/=node_factor; 
                  if (c_scale<1) c_scale=1;
              }             
              /*set up scaled node sizes*/
              for (zw=node_no;zw>=1;zw--){
                 if (no_links==1 && jump_layout_only==0) m_node_size[zw]=c_scale;
                 else{	
                    av_f= m_space / node_no; 
                    if (f_or_n=='f' || jump_layout_only) this_f=(double)m_basin_volume[zw];
                    else if (f_or_n=='a') this_f=(double)histnodesize[zw];
                    else if (f_or_n=='n') this_f=(double)netnodesize[zw];
	            m_node_size[zw]=c_scale*(float)pow(this_f/av_f,.5);
		 }   
              } 
              draw_box(-1,X1,Y1,X2,Y2,1,0);/*clear box*/       
              draw_jump_graph(c_scale,m_node_size,wox,woy,X1,Y1,X2,Y2);
              break;            
           case 'E': case 'C': case 'B': case 'b':{ /*expand/contract links, 
                                                     or both links and nodes*/ 
              float this_angle,this_rad;              
              if (is[0]=='B') c_scale*=node_factor;
              else if (is[0]=='b'){
                  c_scale/=node_factor;
                  if (c_scale<1) c_scale=1;
              }
              if (is[0]=='B' || is[0]=='b'){
                 /*set up scaled node sizes*/
                 for (zw=node_no;zw>=1;zw--){
                    if (no_links==1  && jump_layout_only==0) m_node_size[zw]=c_scale;
                    else{	
                       av_f= m_space / node_no; 
                       if (f_or_n=='f' || jump_layout_only) this_f=(double)m_basin_volume[zw];
                       else if (f_or_n=='a') this_f=(double)histnodesize[zw];
                       else if (f_or_n=='n') this_f=(double)netnodesize[zw];
	               m_node_size[zw]=c_scale*(float)pow(this_f/av_f,.5);
                    }    
                 }   
              }
              if (is[0]=='E' || is[0]=='B') w_cyclerad*=link_factor;
              else if (is[0]=='C' || is[0]=='b') w_cyclerad/=link_factor; 
	      
              for (zw=node_no;zw>=1;zw--){
		 compute_link_angle(wox,woy,basin_pos[zw][0],
				            basin_pos[zw][1]);
                 this_angle=linkangle;
  
                 if (!(wox==basin_pos[zw][0] && woy==basin_pos[zw][1])){
                    this_rad=pow((pow(abs(wox-basin_pos[zw][0]),2)+
                                pow(abs(woy-basin_pos[zw][1]),2)),.5);
                    if (is[0]=='E' || is[0]=='B') this_rad*=link_factor;
                    else if (is[0]=='C' || is[0]=='b') this_rad/=link_factor;
                    basin_pos[zw][0]=(cos(-(this_angle))*this_rad)+wox;
		    basin_pos[zw][1]=(sin(-(this_angle))*this_rad)+woy;
	         }
	      }             
              draw_box(-1,X1,Y1,X2,Y2,1,0);/*clear box*/
              draw_jump_graph(c_scale,m_node_size,wox,woy,X1,Y1,X2,Y2);
              break;      
           }           
	   case '1':{ /*1d layout, set and remember initial basin positions*/
              double gap; 
                c_scale=PI*w_cyclerad/(node_no*1.2);
                if (c_scale>w_cyclerad/5) c_scale=w_cyclerad/5; 
                if (node_no>100) /*revose for bigger networks*/
                     c_scale*=(node_factor*(pow(node_no,0.5)/pow(100,0.5)));  
                if (c_scale<2.5) c_scale=2.5;     
                /*1d: set and remember initial basin positions*/
                gap=0.9*(double)(X2-X1)/node_no;

		for (zw=node_no;zw>=1;zw--){
                    if (no_links==1 && jump_layout_only==0) m_node_size[zw]=c_scale;
                    else{
                       av_f= m_space / node_no; 
                       if (f_or_n=='f' || jump_layout_only) this_f=(double)m_basin_volume[zw];
                       else if (f_or_n=='a') this_f=(double)histnodesize[zw];
                       else if (f_or_n=='n') this_f=(double)netnodesize[zw];
	               m_node_size[zw]=c_scale*(float)pow(this_f/av_f,.5);
                    }
		    basin_pos[zw][0]=wox-gap*node_no/2+
                                          gap*(node_no-zw)+gap/2;
                    basin_pos[zw][1]=woy;
                }             
                draw_box(-1,X1,Y1,X2,Y2,1,0);/*clear box*/
                draw_jump_graph(c_scale,m_node_size,wox,woy,X1,Y1,X2,Y2);
                break;
            }
            case '2':{ /*2d layout: set and remember initial basin positions*/
              /*toggle between hex/square*/
              graph_hex2d=abs(graph_hex2d-1);
              c_scale=draw_2d_graph(c_scale,m_node_size,X1,Y1,X2,Y2,
                         wox,woy,node_factor);
              break;
            }
            case '3':{ /*3d layout: set and remember initial basin positions*/
               c_scale=draw_3d_graph(c_scale,m_node_size,X1,Y1,X2,Y2,
                         wox,woy,node_factor);	      
              break;
            }
            case 'r': case 'R': 
              /* r - randomize the existing layout by a small amount*/
	      /* R - random layout*/ 
              for (zw=node_no;zw>=1;zw--){
		  if (is[0]=='r'){
                       basin_pos[zw][0]+=
                         (rand()%(vgax(10)) - rand()%(vgax(10)) );
		       basin_pos[zw][1]+=
                         (rand()%(vgax(10)) - rand()%(vgax(10)) );
                  }
                  else if (is[0]=='R'){
		     basin_pos[zw][0]=rand()%(X2-X1-vgax(240)) + X1+vgax(120);
		     basin_pos[zw][1]=rand()%(Y2-Y1-vgay(60)) + Y1+vgay(30);
                  }
	      }
              draw_box(-1,X1,Y1,X2,Y2,1,0);/*clear box*/
              draw_jump_graph(c_scale,m_node_size,wox,woy,X1,Y1,X2,Y2);
              break;
            
	   case 's': /*rescale basins for the field inside nodes*/
              if ((f_or_n=='n' && jump_layout_only==0) || f_or_n=='a') goto startbox;
	      xbox=draw_prompt_box(3);
              outtext("change basin scale for graph");
              moveto(xbox,gety+CHeight);
              sprintf(textb[0],"(rad main window=%.1f, in graph=%.1f)", 
		                     jump_startrad,startrad);
              outtext(textb[0]);
              moveto(xbox,gety+CHeight);
              sprintf(textb[0],
                "select attractor rad (min %.2f def %.3g):",220.0/Maxx,startrad);
              outtext(textb[0]);                 
              input_string(4,0);
              if (is[0]!=13 && atof(is)>=220.0/Maxx){
                float r; 
                signed int z;  
                 startrad=atof(is);
                 /*set up array of levels*/
                 for (z=1;z<=lastlevel;z++){    /*lastindex=lastlevel-1*/
                    if (z==1) r=startrad;
                    else r += (startrad/(trans_shrink_fact)/trans_shrink_fact)/log(pow(z,2));
                    rad[z-1]=r;
                 }
	      }
              erase_prompt_box(3);
              goto startprompt;    
           /*redraw the field with basins in new positions
              'I'=inside nodes, 'i' without nodes*/
           case 'i': case 'I':{ 
              float node_size;
              /*if (f_or_n!='f') goto startbox;*/
              if ((f_or_n=='n' && jump_layout_only==0) || f_or_n=='a') goto startbox;
              if (basin_postScript) prepare_basinPS(1,1); /*to draw basin field to postScript*/
              use_preset_basin_pos=1;
              if (is[0]=='I'){ /*empty circles inside nodes*/      
                 for (zw=node_no;zw>=1;zw--){	     
                    /*remembered positions*/
                    moveto(basin_pos[zw][0],basin_pos[zw][1]);
                    setfillstyle(SOLID_FILL,black);
                    if (scale_m_nodes==1)
	                 node_size=0.9*m_node_size[zw];    
                    else node_size=0.9*c_scale;  
                    pieslice(1,getx,gety,0,360,(signed int)node_size);

                    if (basin_postScript){ /*empty circles in postscript*/
                       color_ps(basin_postScript-1,black);
                       circle_fill_ps(basin_pos[zw][0],basin_pos[zw][1],
                                      (double)node_size);/*filled circle*/
                       color_ps(basin_postScript-1,white);
                       circle_ps(basin_pos[zw][0],basin_pos[zw][1],
                                      (double)node_size);/*outline*/
                    }
	         }
              }
              else if (is[0]=='i') draw_box(-1,X1,Y1,X2,Y2,1,0);/*clear box*/
              erase_prompt_box(4);
              
              draw_field(); /*redraw the field with basins in new positions*/
              if (basin_postScript) close(fid_PS);/*close the postScript file -- for DOS to work*/
              print_field_data();

              input_string(1,0);
              /*note: to save basins inside jumpgraph nodes as a postPcript file,
                      in the directory with the .ps files, enter the following in xterm,
                      concatenate: cat my_jgPS.ps my_bPS.ps > temp.ps or similar,
                      first the jumpgraph.ps file, then the basin.ps file*/ 
              draw_box(-1,X1,Y1,X2,Y2,1,0);/*clear box*/
              draw_jump_graph(c_scale,m_node_size,wox,woy,X1,Y1,X2,Y2);
              is[0]=0;
              if (basin_postScript) prepare_basinPS(1,1); /*to reset postScript*/
              goto startprompt;          
	    }
          }

	  if (is[0]!='q') goto startbox;
          /*for space-time patterns according to graph*/
          if (att_types==1 && f_or_n=='n'){
	     for (zw=0;zw<=node_no;zw++) /*copy node positions*/
	       memmove(stp_graph_layout[zw],basin_pos[zw],
                                          sizeof(signed int)*2);
             /*acopy node sizes*/                      
             memmove(stp_graph_node_size,m_node_size,(node_no+1)*sizeof(float));
          }
          free(reorder_graph);
          for (zw=0;zw<=node_no;zw++) free(temp_basin_pos[zw]);
          free(temp_basin_pos);
          /*free basin positions, and scales node sizes*/
          for (zw=0;zw<=node_no;zw++) free(basin_pos[zw]);
          free(basin_pos);         
          free(m_node_size);
          free(outside_inputs);
          sprintf(textb[9],"%s","boximage[3]");   
          putimage(X1,Y1,boximage[3],COPY_PUT,textb[9]);
          uncover_free_prompt_box(3); 
    }/******* end: draw jump-graph *****/

    if (no_links==0){
          for (zw=1;zw<=node_no;zw++){ 
             free(basinjumps[zw]);
             free(b_j[zw]);
          }
          free(basinjumps);
          free(b_j);
    }
    free(basin_color);
    if (f_or_n=='f' || jump_layout_only==1){
          for (zz=0;zz<=node_no;zz++){                 
            for (z=0;z<=m_att_period[zz];z++) free(m_att[zz][z]);
          }
          for (zz=0;zz<=node_no;zz++) free(m_att[zz]);
          free(m_att);
    }
    if (jump_layout_only==1) free(m_basin_volume);          
    else if (no_links==0){
          free(m_att_period);
          if (f_or_n=='f') free(m_basin_volume);
          else if (f_or_n=='a') free(histnodesize);
          else if (f_or_n=='n') free(netnodesize);
    }

    { float r; 
         signed int z; 
          startrad=jump_startrad;
          /*set up array of levels*/
          for (z=1;z<=lastlevel;z++){    /*lastindex=lastlevel-1*/
             if (z==1) r=startrad;
	     else    r += startrad/log(pow(z,2));
             rad[z-1]=r;
          }
     }

     datapause=jump_datapause; /*restore datapause for main basins;*/
  
     xoffset=temp_xoffset;
     yoffset=temp_yoffset;
     if (jump_graph==2 || jump_graph==3 || jump_graph==4)
          fprintf(stderr,"\n ------------------------------------------");
     return(1); /*graph completed, otherwise return(0)*/
}

/******* circle layout: set and remember initial basin positions *******/
float draw_circle_graph(float c_scale,float *m_node_size,
              signed int X1, signed int Y1,
              signed int X2, signed int Y2,
              signed int wox, signed int woy,
              float node_factor,double w_angle){

  double this_f, av_f;
  unsigned int zw;

              c_scale=PI*w_cyclerad/(node_no*1.2);
              if (c_scale>w_cyclerad/5) c_scale=w_cyclerad/5;
              if (node_no>100)
	            c_scale*=(node_factor*(pow(node_no,0.5)/pow(100,0.5)));
              if (c_scale<2.5) c_scale=2.5;
              /*circle: set and remember initial basin positions*/
              /*fprintf(stderr,"\n (node_no=%d c_scale=%.1f) ",node_no, c_scale);*/ 
              for (zw=node_no;zw>=1;zw--){
	         if (no_links==1 && jump_layout_only==0) m_node_size[zw]=c_scale;
		 else{
                    av_f= m_space / node_no; 
                    if (f_or_n=='f' || jump_layout_only){
                       this_f=(double)m_basin_volume[zw];
      /*fprintf(stderr,"\n m_basin_volume[%d]=%lu m_space=%.1f node_no=%d  av_f=%.1f this_f=%.1f ",
	zw,m_basin_volume[zw],m_space, node_no,av_f,this_f);*/
                    }
                    else if (f_or_n=='a') this_f=(double)histnodesize[zw];
                    else if (f_or_n=='n') this_f=(double)netnodesize[zw];
	            m_node_size[zw]=c_scale*(float)pow(this_f/av_f,.5);
                    /*fprintf(stderr,"m_node_size[%d]=%.1f ",zw,m_node_size[zw]);*/ 
		 }
                 basin_pos[zw][0]=(cos(-(w_angle*zw))*w_cyclerad)+wox;
                 basin_pos[zw][1]=-(sin(-(w_angle*zw))*w_cyclerad)+woy;
               }
              draw_box(-1,X1,Y1,X2,Y2,1,0);/*clear box*/
              draw_jump_graph(c_scale,m_node_size,wox,woy,X1,Y1,X2,Y2);
              return(c_scale);
}

/****** 2d layout: set and remember initial basin positions *******/
float draw_2d_graph(float c_scale,float *m_node_size,
              signed int X1, signed int Y1,
              signed int X2, signed int Y2,
              signed int wox, signed int woy,float node_factor) {

  double this_f, av_f;
  signed int tempi2d,i2d_m,j2d_m,xz,yz;
  unsigned int poss;
  double gap;

              /*reset node scale*/
              c_scale=PI*w_cyclerad/(node_no*1.2);
              if (c_scale>w_cyclerad/5) c_scale=w_cyclerad/5;
              if (node_no>100)
                 c_scale*=(node_factor*(pow(node_no,0.7)/pow(100,0.7)));
              if (c_scale<2.5) c_scale=2.5; 
              /*To represent jumpgraph roughly 2d*/ 
              if (f_or_n=='f' || f_or_n=='a' || oned || threed)
		i2d_m=j2d_m=(signed int)ceil(pow((double)node_no,.5));
              else if (twod){
	         i2d_m=i2d;
	         j2d_m=j2d;
	      }
	      else if (threed){
	         i2d_m=i3d;
	         j2d_m=j3d*h3d;
	      }
              /*make sure i is bigger*/
              if (!twod && f_or_n=='n' && j2d_m>i2d_m){ 
                 tempi2d=i2d_m;
                 i2d_m=j2d_m;
                 j2d_m=tempi2d;
              }
	      gap=2*w_cyclerad/j2d_m;              
              if (i2d_m*gap>X2-X1) gap=(X2-X1)/i2d_m;
              
              for (yz=j2d_m;yz>=1;yz--){
		for (xz=i2d_m;xz>=1;xz--){                                 
                    poss=i2d_m*(yz-1) + xz;
                    if (poss>node_no) continue;
                    /*for node scale*/
                    if (no_links==1 && jump_layout_only==0) m_node_size[poss]=c_scale;
                    else{
                       av_f= m_space / node_no; 
                       if (f_or_n=='f' || jump_layout_only) this_f=(double)m_basin_volume[poss];
                       else if (f_or_n=='a') this_f=(double)histnodesize[poss];
                       else if (f_or_n=='n') this_f=(double)netnodesize[poss];
	               m_node_size[poss]=c_scale*(float)pow(this_f/av_f,.5);
                    }
		    basin_pos[poss][0]=(wox-gap*i2d_m/2)+gap*(i2d_m-xz)+gap/2;
                    basin_pos[poss][1]=(woy-gap*j2d_m/2)+gap*(j2d_m-yz)+gap/2;
                 }
              }
              /*show hex layout triangular grid for 2d CA, even sides, and k=6 or 7*/
              /*if (f_or_n=='n' && twod && nhood_mixed==0 
		        && (nhood==6 || nhood==7) && i2d%2==0 && j2d%2==0){*/
                if (graph_hex2d==1){
		    double gap60;
                    gap60=gap/cos(PI/6); /*PI/2=90 degrees*/                   
                    for (yz=j2d_m;yz>=1;yz--){
		       for (xz=i2d_m;xz>=1;xz--){
                          poss=i2d_m*(yz-1) + xz;
                          if (poss>node_no) continue;
                          
                          if (yz%2==1)
		             basin_pos[poss][0]=(wox-gap60*i2d_m/2)+
                                          gap60*(i2d_m-xz)+gap60;
                          else basin_pos[poss][0]=(wox-gap60*i2d_m/2)+
                                          gap60*(i2d_m-xz)+gap60/2;
                          basin_pos[poss][1]=
                            (woy-gap*j2d_m/2)+gap*(j2d_m-yz)+gap/2;
                       }
                    }
              } 
              draw_box(-1,X1,Y1,X2,Y2,1,0);/*clear box*/
              draw_jump_graph(c_scale,m_node_size,wox,woy,X1,Y1,X2,Y2);
              return(c_scale);
}

/******* 3d layout: set and remember initial basin positions *******/
float draw_3d_graph(float c_scale,float *m_node_size,
              signed int X1, signed int Y1,
              signed int X2, signed int Y2,
              signed int wox, signed int woy, float node_factor) {

 double this_f, av_f;
 signed int i3d_m,j3d_m,h3d_m,xz,yz,hz;
 unsigned int poss;
 double gap;
            
              /*reset node scale*/
              c_scale=PI*w_cyclerad/(node_no*1.2);
              if (c_scale>w_cyclerad/5) c_scale=w_cyclerad/5;
              if (node_no>100)
                 c_scale*=(node_factor*(pow(node_no,0.7)/pow(100,0.7)));
              if (c_scale<2.5) c_scale=2.5;
              /*To represent jumpgraph roughly 3d*/
              if (f_or_n=='f' || f_or_n=='a' || threed==0)
		i3d_m=j3d_m=h3d_m=
                       (signed int)ceil(pow((double)node_no,1.0/3));
              else if (threed){
	         i3d_m=i3d;
	         j3d_m=j3d;
                 h3d_m=h3d;
	      }
	      gap=w_cyclerad/h3d_m;  
              if (i3d_m*gap>X2-X1) gap=(X2-X1)/i3d_m;            
              for (hz=h3d_m;hz>=1;hz--){
                 for (yz=j3d_m;yz>=1;yz--){
		    for (xz=i3d_m;xz>=1;xz--){
                       poss=i3d_m*j3d_m*(hz-1)+i3d_m*(yz-1)+xz;
                       if (poss>node_no) continue;
                       /*for node scale*/
                       if (no_links==1 && jump_layout_only==0) m_node_size[poss]=c_scale;
                       else{
                          av_f= m_space / node_no;  
                          if (f_or_n=='f' || jump_layout_only) this_f=(double)m_basin_volume[poss];
                          else if (f_or_n=='a') 
                                this_f=(double)histnodesize[poss];
                          else if (f_or_n=='n') this_f=
                                (double)netnodesize[poss];
	                  m_node_size[poss]=c_scale*(float)pow(this_f/av_f,.5);
                       }
                       basin_pos[poss][0]=
                                   wox+gap*(i3d_m-xz)+gap/2-sin(PI/4)*yz*gap;
                       basin_pos[poss][1]=(woy-0.5*gap*h3d_m*(1+sin(PI/4)))
                                    +gap*(h3d_m-hz)+gap/2+sin(PI/4)*yz*gap;
                    }
                 }
              }
              draw_box(-1,X1,Y1,X2,Y2,1,0);/*clear box*/
              draw_jump_graph(c_scale,m_node_size,wox,woy,X1,Y1,X2,Y2);
              return(c_scale);
}

/********************** unscramble the graph  ************************/
void unscramble_graph(signed int wox, signed int woy){
  unsigned int zw,zu,*node_outputs,*node_inputs,*shuford,
                             con_to,ord,bytes_size;
  signed int dist_from_center;      
  float this_angle,rand_angle1,rand_angle2,this_rad;
  char *in_check,*out_check,in_out_order,count=0;
  
  if (no_links==1) return;
 
        if ((node_outputs=(unsigned int*)calloc(node_no+1,
                  sizeof(unsigned int)))==NULL) outof_memory("node_outputs");    
    
        if ((node_inputs=(unsigned int *)calloc(node_no+1,
                   sizeof(unsigned int)))==NULL) outof_memory("node_inputs");
        /*shuffle order*/
        if ((shuford=(unsigned int *)calloc(node_no+1,
                   sizeof(unsigned int)))==NULL) outof_memory("shuford");

        bytes_size=(signed int)ceil((double)(node_no+1)/8);
        if ((out_check=(char *)calloc(bytes_size,sizeof(char)))
					==NULL) outof_memory("out_check");
        if ((in_check=(char *)calloc(bytes_size,sizeof(char)))
					==NULL) outof_memory("in_check");

        for (zu=1;zu<=node_no;zu++){
                for (zw=1;zw<=node_no;zw++){
		   if (zw!=zu ){ /*don't count self outputs*/		     
                      node_outputs[zu]+=basinjumps[zu][zw];
                      node_inputs[zu]+=basinjumps[zw][zu];
                   }
	         }
        }
       
        in_out_order=myrand()%2; /*0 or 1*/
 
        while(count<=2){
	   count+=1;
           if (in_out_order==0){
              shuffle_order1(shuford,node_no);
              for (zu=1;zu<=node_no;zu++){
		 ord=shuford[zu];  /*1 output and wasnt a target before*/
		 if (node_outputs[ord]==1 
                            && get_bit(out_check,ord)==0
                            && get_bit(in_check,ord)==0){
		    for (zw=1;zw<=node_no;zw++){
		       if (basinjumps[ord][zw]>=1){
			   con_to=zw; /*what its connected to*/
                           set_bit1(out_check,con_to);
                           break;
                       }
                    }
                    compute_link_angle(wox,woy,basin_pos[con_to][0],
				              basin_pos[con_to][1]);
                    this_angle=linkangle;
                    rand_angle1=rand()%100 * (PI/6)/100;  /*10 degrees each side*/
		    rand_angle2=rand()%100 * (PI/6)/100;
                    this_rad=w_cyclerad*0.15;
                    dist_from_center=
                      pow((pow(abs(wox-basin_pos[con_to][0]),2)+
		      pow(abs(woy-basin_pos[con_to][1]),2)),.5);
                    /*limit rad*/
                    if (dist_from_center>w_cyclerad*1.5) this_rad*=-1;          
                    
                    basin_pos[ord][0]=
		      (cos(-(this_angle+rand_angle1-rand_angle2))*this_rad)
                                                      +basin_pos[con_to][0];
		    basin_pos[ord][1]=
		      (sin(-(this_angle+rand_angle1-rand_angle2))*this_rad)
		                                      +basin_pos[con_to][1];
	         }
	      }
              in_out_order=1;
           }
           else if (in_out_order==1){
              shuffle_order1(shuford,node_no);
              for (zu=1;zu<=node_no;zu++){
                 ord=shuford[zu];  /*1 output and wasnt a source before*/
                 if (node_inputs[ord]==1 
                              && get_bit(in_check,ord)==0
                              && get_bit(out_check,ord)==0){
		    for (zw=1;zw<=node_no;zw++){
		       if (basinjumps[zw][ord]>=1){
			   con_to=zw; /*what its connected to*/
                           set_bit1(in_check,con_to);
                           break; 
                       }
                    }
                    compute_link_angle(wox,woy,basin_pos[con_to][0],
				              basin_pos[con_to][1]);
                    this_angle=linkangle;
                    rand_angle1=rand()%100 * (PI/6)/100; /*30 degrees each side*/
		    rand_angle2=rand()%100 * (PI/6)/100;
                    
                    this_rad=w_cyclerad*0.15;
                    dist_from_center=
                      pow((pow(abs(wox-basin_pos[con_to][0]),2)+
		      pow(abs(woy-basin_pos[con_to][1]),2)),.5);
                    /*limit rad*/
                    if (dist_from_center>w_cyclerad*1.5) this_rad*=-1;                    
                    basin_pos[ord][0]=
		      (cos(-(this_angle+rand_angle1-rand_angle2))*this_rad)
		                                      +basin_pos[con_to][0];
		    basin_pos[ord][1]=
		    (sin(-(this_angle+rand_angle1-rand_angle2))*this_rad)
                                                      +basin_pos[con_to][1];
		 }                 
	      }
              in_out_order=0;
              
	   }
	}              
        free(node_outputs);
        free(node_inputs);
        free(shuford);
        free(out_check);
        free(in_check);
}

/************************* shuffle the order 1 to n ************************/
void shuffle_order1(unsigned int *shuff_array, unsigned int size){
  unsigned int zu,zz;
             for (zu=1;zu<=size;zu++){
                 reorder:
                 shuff_array[zu]= myrand()%(size+1);
                 if (shuff_array[zu]==0) goto reorder;
                 if (zu>1){
	            for (zz=1;zz<=zu-1;zz++){
	               if (shuff_array[zu]==shuff_array[zz]) goto reorder;
	            }
                 }
              }
}
/***************** reorder according to rank  ************************/
void reorder_rank(unsigned int *reorder_graph){
    unsigned int zw,*in_out_freq,temp_order;
    char sortneeded;

              if (no_links==1) return;

              /*reorder according to rank order*/
              if (f_or_n=='n'){
		 /*for networks, reorder from least to most linked*/
                 if ((in_out_freq=(unsigned int*)
                      calloc(node_no+1,sizeof(unsigned int)))
					==NULL) outof_memory("in_out_freq");
                 /*compute in-out frequency*/
                 for (zw=1;zw<=node_no;zw++)
	            in_out_freq[zw]=nhoodm[zw-1]+celloutfreq[zw-1];
              }
              while(1){
                   sortneeded=0;
                   for (zw=2;zw<=node_no;zw++){
                      if (f_or_n=='n'){
                         /*for networks, reorder from least to most linked*/
                         if (in_out_freq[reorder_graph[zw]]
                               < in_out_freq[reorder_graph[zw-1]]){
		            temp_order=reorder_graph[zw-1];
                            reorder_graph[zw-1]=reorder_graph[zw];
		            reorder_graph[zw]=temp_order;
                            sortneeded=1;
                         }
                      }
		      else if (f_or_n=='f' || f_or_n=='a'){
                        /*for networks, reorder from smallest to biggest basin volume*/
			if (f_or_n=='f'){
                            if (m_basin_volume[reorder_graph[zw]]
                                  < m_basin_volume[reorder_graph[zw-1]]){
		                temp_order=reorder_graph[zw-1];
                                reorder_graph[zw-1]=reorder_graph[zw];
		                reorder_graph[zw]=temp_order;
                                sortneeded=1;
                            } 
                         }
			 else if (f_or_n=='a'){
                            if (histnodesize[reorder_graph[zw]]
                                  < histnodesize[reorder_graph[zw-1]]){
		                temp_order=reorder_graph[zw-1];
                                reorder_graph[zw-1]=reorder_graph[zw];
		                reorder_graph[zw]=temp_order;
                                sortneeded=1;
                            } 
                         }
                      }



		   }
                   if (sortneeded==0) break;
              }
              if (f_or_n=='n') free(in_out_freq);
}

/***************** rotate graph or component ************************/
void rotate_component_graph(char fragment, 
                           char clockwise,float rot_angle,
                           signed int c_x, signed int c_y){
  unsigned int zw;
  float this_angle,this_rad,this_rot_angle;

  if (clockwise=='x') this_rot_angle=-rot_angle;
  else if (clockwise=='X') this_rot_angle=rot_angle; 
              //rotate whole jumpgraph by 10 degrees, or new angle set
              for (zw=node_no;zw>=1;zw--){
		 compute_link_angle(c_x,c_y,basin_pos[zw][0],
				            basin_pos[zw][1]);
                 this_angle=linkangle;
  
                 if (!(c_x==basin_pos[zw][0] && c_y==basin_pos[zw][1])){
		   if (fragment==1 && get_bit(linknet,zw)==0) continue;
                   this_rad=pow((pow(abs(c_x-basin_pos[zw][0]),2)+
                                pow(abs(c_y-basin_pos[zw][1]),2)),.5);
                   basin_pos[zw][0]=(cos(-(this_angle+
                                            this_rot_angle))*this_rad)+c_x;
		   basin_pos[zw][1]=(sin(-(this_angle+
                                            this_rot_angle))*this_rad)+c_y;
	         }
              }
}

/***************** rotate graph or component ************************/
void flip_component_graph(char fragment, char which,
                           signed int c_x, signed int c_y){
  signed int flip;
  unsigned int zw;

              for (zw=node_no;zw>=1;zw--){
                if (fragment==1 && get_bit(linknet,zw)==0) continue;
		if (which=='h'){ 
                    flip=c_x-basin_pos[zw][0];
                    basin_pos[zw][0]+=(flip*2);
                }  
                else if (which=='v'){ 
                    flip=c_y-basin_pos[zw][1];
                    basin_pos[zw][1]+=(flip*2);
                }                 
              }
}

/**************** save or load  graph layout ************************/
char save_load_graph_layout(char save_load){
     unsigned char *graph_pos;
     unsigned int thisbyte,filebytes,zw;
     char file_ret,fid,ok=1;
     
        if ((graph_pos=(unsigned char*)calloc((node_no+1)*4,
                  sizeof(unsigned char)))==NULL) outof_memory("graph_pos");

	if (save_load=='s'){ /*save node  positions*/
              for (zw=1;zw<=node_no;zw++){
		 /*fprintf(stderr,"\n%dx%d ",basin_pos[z][0],basin_pos[z][1]);*/
		 thisbyte=zw*4;                 
		 graph_pos[thisbyte]=(unsigned char)(basin_pos[zw][0] & 0x00ff); 
                 graph_pos[thisbyte+1]=(unsigned char)(basin_pos[zw][0] >> 8);
                 graph_pos[thisbyte+2]=(unsigned char)(basin_pos[zw][1] & 0x00ff); 
                 graph_pos[thisbyte+3]=(unsigned char)(basin_pos[zw][1] >> 8);
              }

              file_ret=select_filename("SAVE","GRAPH LAYOUT","*.grh","my_grh");
              if (file_ret=='q') goto exitFunc;
                 
	      fid=myopen(filename,1,0);
              if (fid==-1){
		 ok=0;
                 file_error(filename);
                 close(fid);
                 goto exitFunc;
              }
              vret=write(fid, graph_pos, (node_no+1)*4*sizeof(unsigned char));
              close(fid);
              goto exitFunc;
        }
        else if (save_load=='l'){ /*load node positions*/
              file_ret=select_filename("LOAD","GRAPH LAYOUT","*.grh","my_grh");
              if (file_ret=='q'){
		   ok=0;
                   goto exitFunc;
              }
	      fid=myopen(filename,0,0);
              filebytes= filelength(fid);
              if (fid==-1 || filebytes!=(node_no+1)*4*sizeof(unsigned char)){
                 ok=0;  
                 if (fid==-1) file_error(filename);
                 else{
                    draw_prompt_box(1);
                    sprintf(textb[1],
                      "can't load: file nodes=%d,  graph nodes=%d, cont-ret:",
		          filebytes/((signed int)(4*sizeof(unsigned char))),
                                                  node_no+1);
                    outtext(textb[1]);
                    input_string(1,0);
                 }
                 close(fid);
                 goto exitFunc;
              }
              vret=read(fid, graph_pos, (node_no+1)*4*sizeof(unsigned char));
              for (zw=1;zw<=node_no;zw++){
                 thisbyte=zw*4;
                 basin_pos[zw][0]=graph_pos[thisbyte]+
                              (((unsigned int)graph_pos[thisbyte+1])<<8);
                 basin_pos[zw][1]=graph_pos[thisbyte+2]+
                              (((unsigned int)graph_pos[thisbyte+3])<<8);
                 /*fprintf(stderr,"\n%dx%d ",basin_pos[z][0],basin_pos[z][1]);*/
              }
              close(fid);      
        }
      exitFunc:
        free(graph_pos); 
        return(ok); 
}

/****** intest paths (directed) or small world distances (non-directed) ******/
/** something wrong here -- excluded for now -- 1 Jan 2010 **/

void intest_path(char what){
  signed int bytesT,xbox;
  unsigned int count_starts,count_starts_temp,
                 zu,dis,snode,fnode,
                 *start_nodes,*start_nodes_temp,**swdis;
  char *d_levels;
  unsigned long matrix_size,in_paths=0,in_path_tsteps=0,
                out_paths=0,out_path_tsteps=0;

   bytesT=(signed int)ceil((double)node_no/8);  
   if ((d_levels=(char*)calloc(bytesT,sizeof(char)))
            ==NULL) outof_memory("d_levels");
   if ((start_nodes=(unsigned int*)calloc(node_no+1,
            sizeof(unsigned int)))==NULL) outof_memory("start_nodes");
   if ((start_nodes_temp=(unsigned int*)calloc(node_no+1,
            sizeof(unsigned int)))==NULL) outof_memory("start_nodes_temp");

   if ((swdis=(unsigned int**)calloc(node_no+1,
            sizeof(unsigned int*)))==NULL) outof_memory("swdis");
   for (zu=1; zu<=node_no; zu++){
      if ((swdis[zu]=(unsigned int*)calloc(node_no+1,
            sizeof(unsigned int)))==NULL) outof_memory("swdis[]");
   }
   
   for (snode=1; snode<=node_no; snode++){ /*start node*/
      /*fprintf(stderr,"\n snode=%d",snode);*/
      count_starts=1;
      start_nodes[1]=snode;
      /*count_bits=1;*/
     
      for (dis=1; dis<=node_no; dis++){ /*final node*/
             count_starts_temp=0;
             for (zu=1; zu<=count_starts; zu++){
	        /*fprintf(stderr," %d", start_nodes[zu]);*/ 
                for (fnode=1; fnode<=node_no; fnode++){ /*final node*/
		  if (what=='p'){ /*directed intest path outputs/inputs*/
                      if (basinjumps[start_nodes[zu]][fnode]>0 && 
		                   get_bit(d_levels,fnode)==0){
		          count_starts_temp+=1;
                          start_nodes_temp[count_starts_temp]=fnode;
	                  swdis[snode][fnode]=dis;
                          set_bit1(d_levels,fnode);
		      }
		  }
                  if (what=='d'){ /*outputs->*/
                      if ((basinjumps[fnode][start_nodes[zu]]>0 ||
			   basinjumps[start_nodes[zu]][fnode]>0 ||
			   start_nodes[zu]==fnode)
                                 && get_bit(d_levels,fnode)==0){
		          count_starts_temp+=1;
                          start_nodes_temp[count_starts_temp]=fnode;
	                  swdis[snode][fnode]=dis;
                          set_bit1(d_levels,fnode);
		      }
		  }     
                }
             }
             memmove(start_nodes,start_nodes_temp,
		    (node_no+1)*sizeof(unsigned int));
             count_starts=count_starts_temp; 
      }
     
      free(d_levels);
      if ((d_levels=(char*)calloc(node_no,sizeof(char)))
				==NULL) outof_memory("d_levels");
   }          
   
   for (snode=1; snode<=node_no; snode++){ /*start node*/ 
         /*fprintf(stderr,"\n %2d.",snode-offset);*/        
         for (fnode=1; fnode<=node_no; fnode++){ /*final node*/
           if (swdis[snode][fnode]>0){
	      in_paths+=1;
              in_path_tsteps+=swdis[snode][fnode];
           }
           if (swdis[fnode][snode]>0){
	      out_paths+=1;
              out_path_tsteps+=swdis[fnode][snode];
           }
         }
   }
   matrix_size=node_no*node_no;
   if (what=='p'){
      sprintf(textb[1],
   "finite intest paths (%lu/%lu=%.2f) av=%.2f",
            out_paths,matrix_size,
                 (float)out_paths/matrix_size,(float)out_path_tsteps/in_paths);
   }
   else if (what=='d'){
      sprintf(textb[1],
         "finite distances (%lu/%lu=%.2f), small world distance (av)=%.2f",
            out_paths-node_no,matrix_size-node_no,
                 (float)(out_paths-node_no)/(matrix_size-node_no)
                       ,(float)(out_path_tsteps-node_no)/in_paths);
   }
   xbox=draw_prompt_box(2);
   outtext(textb[1]);
   moveto(xbox,gety+CHeight);
   outtext("cont-ret:");

   input_string(1,0);

   free(d_levels);
   for (zu=1; zu<=node_no; zu++) free(swdis[zu]);
   free(swdis);
}

/********** prompts for probabilistic ant in jumpgraph  *******************/
void prob_ant_prompts(char show_ant, char rand_ant){
  signed int xbox;

           xbox=draw_prompt_box(3);
           outtext("probabilistic ant: quit-q speed-</> pause-p");
           moveto(xbox,gety+CHeight);
           outtext("restart rnd -r, redraw-d");
           moveto(xbox,gety+CHeight);
           if (show_ant==1) sprintf(textb[1],"%s","ON");
           else sprintf(textb[1],"%s","OFF");
           if (rand_ant==1) sprintf(textb[2],"%s","ON");
           else sprintf(textb[2],"%s","OFF");
           sprintf(textb[0],
               "show hits -h, tog: show ant(%s) -a, rnd ant(%s) -s",
                                  textb[1],textb[2]);
           outtext(textb[0]);
}


/********** launch probabilistic ant in jumpgraph  *******************/
void prob_ant(float c_scale,float *m_node_size,
               signed int wox, signed int woy,
                    signed int X1,signed int Y1,
		       signed int X2,signed int Y2){

  signed int xbox,current_node;
  char test,exitwhile=0, show_ant=1, rand_ant=0, enter_switch=0;
  unsigned char tempcolor;
  unsigned int zw,btarget,wcount=0,*prob_target,*target,pick,range;
  signed int target_no=0,new_target;
  float av_hits,angle,x,y,
    oldx,oldy,dx,dy,distance,start_distance,hit_node_size;
  unsigned long zu,speed,max_speed,*hit_count,hit_total=0,s_space;

   if (no_links==1) return;

  s_space=(unsigned long)(m_space-1); 
 top_start:
   wcount=0;/*reset count*/

   if ((prob_target=(unsigned int *)calloc(node_no+1,
            sizeof(unsigned int)))==NULL) outof_memory("prob_target");
   if ((target=(unsigned int *)calloc(node_no+1,
            sizeof(unsigned int)))==NULL) outof_memory("target");
   /*count hits*/
   if ((hit_count=(unsigned long *)calloc(node_no+1,
            sizeof(unsigned long)))==NULL) outof_memory("(hit_count");

   prob_ant_prompts(show_ant, rand_ant);
  
   tempcolor=getcolor();
   speed=(unsigned long)((unsigned short)-1);
   max_speed=speed*(speed/8);//max loop to slow down
   //fprintf(stderr,"\nspeed=%lu max=%lu",speed,max_speed);
   hit_total=0;

 start:
   /*select node at random*/   
   if (rand_ant==0){ 
      hit_total=0;
      current_node=rand()%node_no+1;
   }
   else{
              /*select node at random, biased by nodesize*/  
              range=0;
              pick=rand()%10000+1;
              for (zw=1; zw<=node_no; zw++){
                 if (f_or_n=='f') range+=(unsigned int)
                        (10000*((double)m_basin_volume[zw]/s_space));
                 else if (f_or_n=='a') range+=(unsigned int)
                        (10000*((double)histnodesize[zw]/s_space));
                 else if (f_or_n=='n') range+=(unsigned int)
                        (10000*((double)netnodesize[zw]/s_space));

                 if (pick<=range){
                    current_node=zw;
                    break;
                 }      
              }
   }

   x=(float)basin_pos[current_node][0];
   y=(float)basin_pos[current_node][1];
   moveto(x,y);

   while(1){
     wcount+=1;
     if (event_ready() || enter_switch){    /*if a key is hit.....*/
        enter_switch=exitwhile=0;
	test=is[0];
	switch(test){
           case 'q':
           /*key 'q' - exit ants*/
           /*clear and redraw graph*/
	   draw_box(-1,X1,Y1,X2,Y2,1,0);/*clear box*/
           draw_jump_graph(c_scale,m_node_size,wox,woy,X1,Y1,X2,Y2);
	   exitwhile=1;
	   break;
           case '>': /*faster*/
	      speed=speed/2;
              if (speed<=1) speed=1;
	      //fprintf(stderr,"\n>speed=%lu",speed);
              break; 
           case '<':  /*slower*/
	      speed=speed*2;
              if (speed>=max_speed) speed=max_speed;
              //fprintf(stderr,"\n>speed=%lu",speed);
              break; 
           case 'p':
              /*key 'p' - pause until another key press
	      this also allows ant to take single steps*/
	      while (!event_ready());
	      break;
           case 'r':
	      /*start at new random node*/
	      wcount=0;
              exitwhile=2;
	      break;
           case 'd':
	      /*clear and redraw graph*/
	      draw_box(-1,X1,Y1,X2,Y2,1,0);/*clear box*/
              draw_jump_graph(c_scale,m_node_size,wox,woy,X1,Y1,X2,Y2);
	      break;
           case 'a':
	      show_ant=abs(show_ant-1);
              prob_ant_prompts(show_ant, rand_ant);
              wcount=0;
	      break;
           case 's':
	      rand_ant=abs(rand_ant-1);
              draw_box(-1,X1,Y1,X2,Y2,1,0);/*clear box*/
              draw_jump_graph(c_scale,m_node_size,wox,woy,X1,Y1,X2,Y2);
              hit_total=0;
              exitwhile=3; /*start new run*/
	      break;
           case 'h':
	      /*show hits*/
	         draw_box(-1,X1,Y1,X2,Y2,1,0);/*clear box*/
                 av_hits=(float)hit_total/node_no;
                 for (zw=node_no;zw>=1;zw--){
                    moveto(basin_pos[zw][0],basin_pos[zw][1]);            
	            hit_node_size=c_scale*
                        (float)pow((double)hit_count[zw]/av_hits,.5);
                    jump_graph_vertex(zw,hit_node_size,
                        hit_node_size,basin_color[zw],X1,Y1,X2,Y2);          
                 }
	        start_hit_prompts:
                 xbox=draw_prompt_box(3);
                 if (f_or_n=='n'){ 
                    outtext("showing % hits on each node");
                    if (target_no==0){ 
                       sprintf(textb[0],
                          " - no targets, ant stuck at node %d",current_node);
                       outtext(textb[0]);
                    }
                 }
                 else { 
                    outtext("showing % hits on each basin");
                    if (target_no==0){ 
                       sprintf(textb[0],
                          " - no targets, ant stuck at basin %d",current_node-1);
                       outtext(textb[0]);
                    }
                 }
                 
                 moveto(xbox,gety+CHeight);
                 sprintf(textb[0],"total hits so far=%lu",hit_total);
                 outtext(textb[0]);
                 moveto(xbox,gety+CHeight);
                 outtext("show % frequency -n noNodes-N, quit-q cont-ret:");
                 input_string(1,0);
                 if (is[0]=='q'){
                    /*clear and redraw graph*/
	            draw_box(-1,X1,Y1,X2,Y2,1,0);/*clear box*/
                    draw_jump_graph(c_scale,m_node_size,wox,woy,X1,Y1,X2,Y2);            
	            exitwhile=1;
	            break;
                 }
                 
                 if (is[0]=='n' || is[0]=='N'){
		   signed int wbox;
                    draw_box(-1,X1,Y1,X2,Y2,1,0);/*clear box*/
                    for (zw=node_no;zw>=1;zw--){
                       if (is[0]=='n'){
                          moveto(basin_pos[zw][0],basin_pos[zw][1]);
	                  hit_node_size=c_scale*
                              (float)pow((double)hit_count[zw]/av_hits,.5);
                         jump_graph_vertex(zw,hit_node_size,
                             hit_node_size,basin_color[zw],X1,Y1,X2,Y2);
                       }
                       moveto(basin_pos[zw][0],basin_pos[zw][1]);
               
                       if (hit_count[zw]==0){ 
                          sprintf(textb[0],"0");
                          wbox=CWidth+CWidth/3;
                       } 
                       else{
                          sprintf(textb[0],"%.1f",
                            100*(float)hit_count[zw]/hit_total);
                          wbox=textwidth(textb[0])-CWidth/3; 
                       }
                       moverel(CWidth/2,0); 
                       setfillstyle(SOLID_FILL,black);
                       bar3d(getx-CWidth/3,gety,
                         getx-CWidth/3+wbox,gety+CHeight,0,0);
                       outtext(textb[0]);                    
                    }
                    goto start_hit_prompts;
                 }              
                 draw_box(-1,X1,Y1,X2,Y2,1,0);/*clear box*/
                 if (hit_total==Maxlu-1) exitwhile=3; /*restart*/
                 else{             
                    draw_jump_graph(c_scale,m_node_size,wox,woy,X1,Y1,X2,Y2);
                    /*restore prompts*/
                    prob_ant_prompts(show_ant, rand_ant);
                 }
                 if (hit_total==Maxlu-1) hit_total=0; /*in case max sample was reached*/
	         break;
	}
        if (exitwhile==1) break;
        if (exitwhile==2) goto start;
        if (exitwhile==3){
           free(prob_target);
           free(target);
           free(hit_count);
           goto top_start;
	}
     }

     if (wcount==1){
        target_no=0; 
        /*calc prob in possible directions*/ 
        for (btarget=1;btarget<=node_no;btarget++){
           if (basinjumps[current_node][btarget]>0){
	      target_no++;
              target[target_no]=btarget;
              prob_target[target_no]=(unsigned int) (10000*
                   ( (double)basinjumps[current_node][btarget]/
		                 (m_att_period[current_node]*m_size) ));
           }           
        }
        /*choose direction*/
        if (target_no>0){ /*in case no targets}*/
           range=0;
           pick=rand()%10000+1;

           for (zw=1; zw<=target_no; zw++){
              range+=prob_target[zw];
              if (pick<=range){
                 new_target=target[zw];
                 break;
              } 
           }     
        }

        if (target_no==0){
           if (rand_ant==0){
              enter_switch=1;
              is[0]='h';
              continue;
           }
           else{ /*pick another start node*/
              wcount=0;
              goto start;
           }
        }

        /*suppress graphics to speed up sample*/
        if (show_ant==0){
	   if (hit_total<Maxlu){
              hit_count[new_target]+=1;
              hit_total+=1;
              //fprintf(stderr," %lu",hit_total);
           }
           wcount=0;/*reset count*/
           if (rand_ant==0) current_node=new_target; 
           else{
              /*select node at random*/ 
	      /*current_node=rand()%node_no+1;*/
              /*select node at random, biased by nodesize*/  
              range=0;
              pick=rand()%10000+1;
              for (zw=1; zw<=node_no; zw++){
                 if (f_or_n=='f') range+=(unsigned int)
                        (10000*((double)m_basin_volume[zw]/s_space));
                 else if (f_or_n=='a') range+=(unsigned int)
                        (10000*((double)histnodesize[zw]/s_space));
                 else if (f_or_n=='n') range+=(unsigned int)
                        (10000*((double)netnodesize[zw]/s_space)); 
                 if (pick<=range){
                    current_node=zw;
                    break;
                 }      
              }       
           }
           if (hit_total==Maxlu-1){ /*in case max sample was reached*/
	       enter_switch=1;               
               is[0]='h';
           } 
           continue;
        } 
        /*set heading*/
        compute_link_angle(basin_pos[current_node][0],
                           basin_pos[current_node][1],
                           basin_pos[new_target][0],
                           basin_pos[new_target][1]);
                    
        angle=linkangle;
       
        start_distance=pow((
           pow(abs(basin_pos[current_node][0]-basin_pos[new_target][0]),2)+
           pow(abs(basin_pos[current_node][1]-basin_pos[new_target][1]),2)),.5);
     }
     
     if (wcount>1){
        /*delete ant*/
        thickline(0,(signed int)oldx,(signed int)oldy,
                   (signed int)(oldx+dx*vgax(8)),
                   (signed int)(oldy+dy*vgay(8)),
                                        white,vgax(2),0);
     }
     oldx=x;
     oldy=y;
  
     /*ant moves forward (acc to where it points)
     by the equivalent of one pixel, by step dx,dy*/		
     dx=cos(angle);
     dy=-sin(angle);
                
     distance=pow((
             pow(abs(basin_pos[current_node][0]-(signed int)x),2)+
             pow(abs(basin_pos[current_node][1]-(signed int)y),2)),.5);
     /*draw ant as thick red? line*/
     thickline(0,(signed int)x,(signed int)y,
               (signed int)(x+dx*vgax(8)),(signed int)y+dy*vgay(8),
			                       lightred,vgax(2),0);
     x+=dx;
     y+=dy;
     /*if ant reaches target, set new origin and target*/	 
     if (abs((signed int)distance)>=
                    abs((signed int)start_distance)-vgax(8)  ||
                  current_node==new_target){
        if (hit_total<=Maxlu){
           hit_count[new_target]+=1;
           hit_total+=1;
           //fprintf(stderr," %lu",hit_total);
        }
        wcount=0;/*reset count*/
        /*redraw target node*/
        moveto(basin_pos[new_target][0],
                           basin_pos[new_target][1]);
        jump_graph_vertex(new_target,m_node_size[new_target],
                      c_scale,basin_color[new_target],X1,Y1,X2,Y2);
        
        if (rand_ant==0) current_node=new_target; 
        else{
              /*select node at random*/ 
	      /*current_node=rand()%node_no+1;*/
              /*select node at random, biased by nodesize*/  
              range=0;
              pick=rand()%10000+1;
              for (zw=1; zw<=node_no; zw++){
                 if (f_or_n=='f') range+=(unsigned int)
                        (10000*((double)m_basin_volume[zw]/s_space));
                 else if (f_or_n=='a') range+=(unsigned int)
                        (10000*((double)histnodesize[zw]/s_space));
                 else if (f_or_n=='n') range+=(unsigned int)
                        (10000*((double)netnodesize[zw]/s_space)); 
                 if (pick<=range){
                    current_node=zw;
                    break;
                 }      
              }
        }
        x=(float)basin_pos[current_node][0];
        y=(float)basin_pos[current_node][1];
     }
     /*slow down*/
     for (zu=0;zu<=speed;zu++);
   }//while

   free(prob_target);
   free(target);
   free(hit_count);
   setcolor(tempcolor);
}

/********** show jump_table as numbers or fractions *******************/
void show_table(char type, signed int X1, signed int Y1,
		           signed int X2, signed int Y2){

  unsigned int btarget,btype;
  signed int xleft,startV=1,startH=1;
  unsigned char tempcolor;
  char save_list_con_elements=0,/* file for neato */
       xterm_print=0; /*print to xterm*/
  signed int count_list;

   if (no_links==1) return;

       /*to save list of connected elements, 2 columns*/
       /*save_list_con_elements=1;*/
       if (save_list_con_elements==1){
          signed int fid;
          char file_ret;           
 	       file_ret=select_filename("SAVE","DATA","*.dat","my_data");
	       if (file_ret!='q'){
	          memmove(filename_data,filename,13);
                  fid=myopen(filename_data,1,1);
                  if (fid==-1) file_error(filename_data);
	          close(fid);
               }
               if (file_ret=='q' || fid==-1) save_list_con_elements=0;
       }

       tempcolor=getcolor();

       start_prompt:
         draw_prompt_box(1);
#      if __UNIX__
         outtext("scan table: exit-ret xterm-x jump-j reset-s down-d up-u right-r left-l");
#      elif __WATCOM__
         outtext("scan table: exit-ret jump-j reset-s down-d up-u right-r left-l");
#      endif

         if (node_no>=19){
#     if __UNIX__
            XSetFont(D,Gc,smallnonpropfont->fid);
#     elif __WATCOM__
            _setcharsize((signed int)((ceil)((float)(.6*ty1))),
	              (signed int)((ceil)((float)(.6*tx1))));
#     endif
	    CHeight*=.6;
	    CWidth*=.6;
         }
         else if (node_no>=14){
#     if __UNIX__
            XSetFont(D,Gc,mediumnonpropfont->fid);
#     elif __WATCOM__
            _setcharsize((signed int)((ceil)((float)(.8*ty1))),
	              (signed int)((ceil)((float)(.8*tx1))));
#     endif
	    CHeight*=.8;
	    CWidth*=.8;
         }
         else{
#     if __UNIX__ 
            XSetFont(D,Gc,nonpropfont->fid);
#     endif
	 }

        start_table:
         
         draw_box(-1,X1,Y1,X2,Y2,1,0);/*clear box*/   
         setviewport(X1,Y1,X2,Y2,1); /*clip in rectangle*/
         setcolor(lightred);
         moveto(X1+CWidth/2,Y1+CHeight/2);
         xleft=getx; 

         /*print numbers*/
         if (type=='t'){
	    if (xterm_print==1) fprintf(stderr,"\n\n     ");
	    moveto(xleft+CWidth*4,gety);
            for (btype=startH;btype<=node_no+1;btype++){
               moveto(xleft+CWidth*4*(btype-startH+1),gety);
               if (btype<=node_no){
	          if (f_or_n=='n') sprintf(textb[0],"%4d",btype-1);
                  else sprintf(textb[0],"%4d",btype);
                  outtext(textb[0]);
                  if (xterm_print==1)  fprintf(stderr,"%s",textb[0]);
               }
               else{
                  if (f_or_n=='n'){
                     sprintf(textb[0],"    out  k");
                     sprintf(textb[1],"    out  k");
                  }
                  else{ 
                     sprintf(textb[0],"      P   J     Volume     Self");
                     sprintf(textb[1],"      P       J          Volume        Self");
                  }
                  outtext(textb[0]);
                  if (xterm_print==1)  fprintf(stderr,"%s",textb[1]);
               }
               
            }            
            for (btype=startV;btype<=node_no;btype++){
               if (f_or_n=='n') sprintf(textb[0],"%4d:",btype-1);
               else sprintf(textb[0],"%4d:",btype);
               moveto(xleft,gety+CHeight);
               outtext(textb[0]);
               if (xterm_print==1) fprintf(stderr,"\n%s",textb[0]);
               for (btarget=startH;btarget<=node_no+1;btarget++){
                 moveto(xleft+CWidth*4*(btarget-startH+1),gety);
		 if (btarget<=node_no){
		       setcolor(white);
                       if (b_j[btype][btarget]==0){
			   outtext("   .");
                           if (xterm_print==1) fprintf(stderr,"   .");
                       }
                       else{
                          if (save_list_con_elements==1){
                             fprintf(stderr,"%d %d\n",btype,btarget);
                             sprintf(textb[0],"%d %d\n",btype,btarget);
                             save_data(textb[0]);
                             count_list+=1;
                          }
                          sprintf(textb[0],"%4lu",b_j[btype][btarget]);
                          outtext(textb[0]);
                          if (xterm_print==1) fprintf(stderr,"%s",textb[0]);
                       }
                       setcolor(lightred);  
		 }
                 else{
                     if (f_or_n=='n'){
                        sprintf(textb[0],"    %d    %d",
			       m_att_period[btype],netnodesize[btype]);
                        outtext(textb[0]);
                        if (xterm_print==1) fprintf(stderr,"%s",textb[0]);
                     }
                     else{
                        sprintf(textb[0],"%7d",m_att_period[btype]);
                        outtext(textb[0]);
                        if (xterm_print==1) fprintf(stderr,"%s",textb[0]);
                        sprintf(textb[0],"%9d",m_att_period[btype]*m_size);
                        outtext(textb[0]);
                        if (xterm_print==1) fprintf(stderr,"%s",textb[0]);
                        if (f_or_n=='f') sprintf(textb[0],"%10lu=%.2f%%",
			       m_basin_volume[btype],
                                    (double)(m_basin_volume[btype]*100)/m_space);
                        else if (f_or_n=='a') sprintf(textb[0],"%10lu=%.2f%%",
				histnodesize[btype],(double)(histnodesize[btype]*100)/m_space);
                        outtext(textb[0]);
                        if (xterm_print==1) fprintf(stderr,"%s",textb[0]);

                        sprintf(textb[0],"%10.2f%%",
				(double)(b_j[btype][btype]*100)/(m_att_period[btype]*m_size));
                        outtext("         ");
                        outtext(textb[0]);
                        if (xterm_print==1) fprintf(stderr,"%s",textb[0]);
                     }
                  }
	       }
            }
         }
         if (save_list_con_elements==1)
                     fprintf(stderr,"total links=%d\n",count_list);
         /*print fractions*/
         if (type=='T'){
            if (xterm_print==1) fprintf(stderr,"\n\n     ");
            moveto(xleft+CWidth*5,gety);
            for (btype=startH;btype<=node_no+1;btype++){
               moveto(xleft+CWidth*5*(btype-startH+1),gety);
               if (btype<=node_no){
	            if (f_or_n=='n') sprintf(textb[0],"%5d",btype-1);
                    else sprintf(textb[0],"%5d",btype);
                    outtext(textb[0]);
                    if (xterm_print==1) fprintf(stderr,"%s",textb[0]);
               }
               else{
                  if (f_or_n=='n'){
                     sprintf(textb[0],"    out  k");
                     sprintf(textb[1],"    out  k");
                  }
                  else{ 
                     sprintf(textb[0],"      P   J     Volume   Self");
                     sprintf(textb[1],"      P       J          Volume       Self");
                  }
                  outtext(textb[0]);
                  if (xterm_print==1) fprintf(stderr,"%s",textb[1]);
               }               
            }
            for (btype=startV;btype<=node_no;btype++){
               if (f_or_n=='n') sprintf(textb[0],"%4d:",btype-1);
               else sprintf(textb[0],"%4d:",btype);
               moveto(xleft,gety+CHeight);
               outtext(textb[0]);
               if (xterm_print==1) fprintf(stderr,"\n%s",textb[0]);
               for (btarget=startH;btarget<=node_no+1;btarget++){
                  moveto(xleft+CWidth*5*(btarget-startH+1),gety);
		  if (btarget<=node_no){
                    setcolor(white); 
		    if (b_j[btype][btarget]==0){
		       outtext("    .");
                       if (xterm_print==1) fprintf(stderr,"    .");
                    }
                    else if (b_j[btype][btarget]==
			     m_att_period[btype]*m_size){
		       outtext("    1");
                       if (xterm_print==1) fprintf(stderr,"    1");
                    }
		    else{
                       sprintf(textb[0],"%.3f",
                         (float)b_j[btype][btarget]/
                                     (m_att_period[btype]*m_size));
                       sprintf(textb[1]," %s",textb[0]+1);
                       outtext(textb[1]);
                       if (xterm_print==1) fprintf(stderr,"%s",textb[1]);
                    }
                    setcolor(lightred);
                  }
                  else{
                     if (f_or_n=='n'){
                        sprintf(textb[0],"    %d    %d",
			       m_att_period[btype],netnodesize[btype]);
                        outtext(textb[0]);
                        if (xterm_print==1) fprintf(stderr,"%s",textb[0]);
                     }
                     else{
                        sprintf(textb[0],"%7d",m_att_period[btype]);
                        outtext(textb[0]);
                        if (xterm_print==1) fprintf(stderr,"%s",textb[0]);
                        sprintf(textb[0],"%9d",m_att_period[btype]*m_size);
                        outtext(textb[0]);
                        if (xterm_print==1) fprintf(stderr,"%s",textb[0]);
                        if (f_or_n=='f') sprintf(textb[0],"%10lu=%.2f%%",
			     m_basin_volume[btype],(double)(m_basin_volume[btype]*100)/m_space);
                        else if (f_or_n=='a') sprintf(textb[0],"%10lu=%.2f%%",
				 histnodesize[btype],(double)(histnodesize[btype]*100)/m_space);
                        outtext(textb[0]);
                        if (xterm_print==1) fprintf(stderr,"%s",textb[0]);

                        sprintf(textb[0],"%10.2f%%",
				(double)(b_j[btype][btype]*100)/(m_att_period[btype]*m_size));
                        outtext("         ");
                        outtext(textb[0]);
                        if (xterm_print==1) fprintf(stderr,"%s",textb[0]);     
                     }
                  }
	       }
            }
         }
	 xterm_print=0;      
         while(event_ready()==0);               
	 switch(is[0]){
              case 'x':
                xterm_print=1;
                goto start_table;      
	      case 's':
		 startV=1;
                 startH=1;
                 goto start_table;
	      case 'j':
                 setcolor(white);
                 restore_def_font();
                 setviewport(0,0,Maxx,Maxy,0);
                 draw_prompt_box(1);                 
                 outtext("jump: row:");
                 input_string(5,0);
                 if (is[0]!=13 && atoi(is)>=1 && atoi(is)<=node_no)
                                 startV=atoi(is);
                 outtext("  column:");         
                 input_string(5,0);
                 if (is[0]!=13 && atoi(is)>=1 && atoi(is)<=node_no)
                                 startH=atoi(is);  
                 goto start_prompt;       
	      case 'd': 
                 startV+=1;
                 if (startV>node_no) startV=node_no;
                 goto start_table; 
	      case 'u':
                 startV-=1;
                 if (startV<=1) startV=1;
                 goto start_table; 
	      case 'r': 
                 startH+=1;
                 if (startH>node_no) startH=node_no;
                 goto start_table; 
	       case 'l':
                   startH-=1;
                   if (startH<=1) startH=1;
                   goto start_table; 
            
	 }

         setcolor(tempcolor);
         restore_def_font();
         setviewport(0,0,Maxx,Maxy,0);
         
}

/********** draw links and ring of vertices, Aug 2000 *******************/
void draw_jump_graph(float c_scale,float *m_node_size,
                       signed int wox, signed int woy,
                         signed int X1,signed int Y1,
			  signed int X2,signed int Y2){

  unsigned int zw, btarget;
  signed int outerx,outery;
  unsigned char tempcolor,thiscolor,startcolor;
  double thisthick,maxthick;
  float self_length;

     setviewport(X1,Y1,X2,Y2,1); /*clip in rectangle*/        
     tempcolor=getcolor();

     if (bground=='w') startcolor=red;
     else startcolor=blue;
     thiscolor=startcolor;
     for (zw=node_no;zw>=1;zw--){
        /*fprintf(stderr,"\n%d:",zw-1);*/
        /*change color for each new attractor cycling through 6 colors*/
        if (thiscolor==lightred) thiscolor=lightcyan;
	 else if (thiscolor==lightcyan) thiscolor=green;
         else if (thiscolor==green) thiscolor=lightmagenta;
	 else if (thiscolor==lightmagenta) thiscolor=yellow;
         else if (thiscolor==yellow) thiscolor=startcolor;
         else if (thiscolor==startcolor) thiscolor=lightred;
        setcolor(thiscolor);
        basin_color[zw]=thiscolor;

	if (no_links==1) goto skip;

        if (scale_m_nodes==1){ 
            maxthick=m_node_size[zw]*.8;
            self_length=m_node_size[zw]*2;
        }
        else{ 
           maxthick=c_scale*.8;
           self_length=c_scale*2;
        }
        for (btarget=node_no;btarget>=1;btarget--){
	   /* draw_jump_lines */
           if (basinjumps[zw][btarget]>0){
	     if (suppress_u_links==1 && zw!=btarget &&
                (outside_inputs[btarget]<=u_threshold ||     
                   outside_inputs[zw]<=u_threshold)) continue;
              
              thisthick=(double)maxthick*(double)basinjumps[zw][btarget]/
		                                (m_att_period[zw]*m_size);
              /*fprintf(stderr,"%d/%d ",
                       basinjumps[zw][btarget],
		       m_att_period[zw]*m_size);*/

              if (zw==btarget){
                 /*jumping to itself, same basin*/
		 compute_link_angle(wox,woy,basin_pos[zw][0],basin_pos[zw][1]);
                 /*find outer positions*/
                 outerx=basin_pos[zw][0]+cos(linkangle)*self_length;
                 outery=basin_pos[zw][1]-sin(linkangle)*self_length;
                 if (scale_links==0) thisthick=maxthick=1;
                 thickline(1,basin_pos[zw][0],basin_pos[zw][1],outerx,outery,
		                thiscolor,thisthick,maxthick);
		 
             }
	     else{
                /*juming to another basin*/
                if (scale_links==0) thisthick=maxthick=1;
                thickline(0,basin_pos[zw][0],basin_pos[zw][1],
		     basin_pos[btarget][0],basin_pos[btarget][1],
                                        thiscolor,thisthick,maxthick);
             }  
          }	   
        }
        skip:;
     }

     setcolor(white);

     /*draw vertices, disk or points*/
     for (zw=node_no;zw>=1;zw--){
        moveto(basin_pos[zw][0],basin_pos[zw][1]);                      
        jump_graph_vertex(zw,m_node_size[zw],
                  c_scale,basin_color[zw],X1,Y1,X2,Y2);
	setcolor(white);
     }
          
     setcolor(tempcolor);
     setviewport(0,0,Maxx,Maxy,0);
}

/****** draw a jump-graph vertex or node as a disk or point, Aug 2000 ********/
void jump_graph_vertex(signed int zw, float node_size,
               float c_scale, unsigned char thiscolor,
                 signed int X1,signed int Y1,
		    signed int X2,signed int Y2){
  float this_node_size;
  unsigned char tempcolor;
  signed int tempx,tempy;

           setviewport(X1,Y1,X2,Y2,1); /*clip in rectangle*/ 
           tempcolor=getcolor();
           tempx=getx;
           tempy=gety;
           if (c_scale<=1){
               putpixel(getx,gety,thiscolor);
                if (graph_postscript){
                   color_ps(graph_postscript-1,thiscolor);
                   smallspot_ps(getx,gety,1);
                }
               goto skip_vertex;
           }
           setfillstyle(SOLID_FILL,thiscolor);
           if (scale_m_nodes==1) this_node_size=node_size;   
           else this_node_size=c_scale;  
      /*fprintf(stderr,"\n this_node_size=%.1f scale_m_nodes=%d, jump_graph=%d jump_layout_only=%d",
	this_node_size, scale_m_nodes, jump_graph, jump_layout_only);*/
           pieslice(1,getx,gety,0,360,(signed int)this_node_size);
           if (graph_postscript){
              color_ps(graph_postscript-1,thiscolor);
              circle_fill_ps(tempx,tempy,(double)this_node_size);/*filled circle*/
              color_ps(graph_postscript-1,white);
              circle_ps(tempx,tempy,(double)this_node_size);/*outline*/
           }
           if (this_node_size>=CHeight*.7 &&  node_numbers==1){
              setcolor(black);                       
              moveto(getx-CWidth/2,gety-CHeight/2);
              if (f_or_n=='n' && jump_layout_only==0) sprintf(textb[1],"%d",zw-1);
              else sprintf(textb[1],"%d",zw);
              outtext(textb[1]);
              if (graph_postscript){
                 color_ps(graph_postscript-1,black);
                 nodetext_ps(2,0,textb[1],tempx,tempy);
              }
           }
          skip_vertex:
           setcolor(tempcolor);
           if (graph_postscript) color_ps(graph_postscript-1,tempcolor);
           setviewport(0,0,Maxx,Maxy,0);
}

/******************* line with varying thickness ***************/
void thickline(char selflink,signed int x1, signed int y1,
	       signed int x2, signed int y2, 
                     unsigned char thiscolor, 
                     double thisthick, double maxthick){ 

  float xgap,ygap,xend,yend;
  char signx,signy,tempcolor;
  signed int gapfactor;
  signed int arrow[6];

         tempcolor=getcolor() ;   
         compute_link_angle(x1,y1,x2,y2);
      
         xend=(float) ( fabs(sin(abs_angle))*thisthick );
         yend=(float) ( fabs(cos(abs_angle))*thisthick );

         /*if (thisthick<=1 || (xend<1 && yend<1) ) thinline=1;*/
         if (selflink==-1){ /*for orbital ant*/
	    selflink=0;
            scale_links=1;
	 }
         /* gap between input and output links*/
         if (scale_links==1) gapfactor=maxthick/7; /*thick links*/
	 else gapfactor=maxthick*2; /*thin links*/
         
	 if (maxthick>=thisthick){
	     xgap=(float) ( fabs(sin(abs_angle))*gapfactor );
	     ygap=(float) ( fabs(cos(abs_angle))*gapfactor );
	 }
         /*make line central ?*/
         else{ 
	    xgap=-(float) ( fabs(sin(abs_angle))*(thisthick/2) );
	    ygap=-(float) ( fabs(cos(abs_angle))*(thisthick/2) );  
         }

	 if ( x1<=x2 && y1>y2){ 
	   signx=-1; signy=-1;            
	 }
         else if (x1>x2 && y1>=y2){
            signx=-1; signy=1; 
	 }
         else if (x1>=x2 && y1<y2){
            signx=1;signy=1;             
	 }
         else if (x1<x2 && y1<=y2){
            signx=1; signy=-1; 
            
	 }
         poly[0]=   x1  +   (xgap*signx);
         poly[1]=   y1  +   (ygap*signy);

         poly[2]=   x2  +   (xgap*signx);
         poly[3]=   y2  +   (ygap*signy);

         setcolor(thiscolor);
         if (graph_postscript) color_ps(graph_postscript-1,thiscolor);
         if (scale_links==1){ /*draw thick link*/
            poly[4]=   poly[2] +  (xend*signx);            
            poly[5]=   poly[3] +  (yend*signy);

            poly[6]=   poly[0] +  (xend*signx);            
            poly[7]=   poly[1] +  (yend*signy);

            /*draw line anyway in case poly too thin, wont show*/            
            line(poly[0],poly[1],poly[2],poly[3]);       
            if (graph_postscript){
              line_block_ps(poly);
              thick_line_ps(.7,poly[0],poly[1],poly[2],poly[3]);   
            }
            /*draw the filled poly*/
	    setfillstyle(SOLID_FILL,thiscolor);           
            fillpoly(4, poly);
	 }
         /*draw thin link*/
         else{ 
            line(poly[0],poly[1],poly[2],poly[3]);
            if (graph_postscript) 
                thick_line_ps(.7,poly[0],poly[1],poly[2],poly[3]);
                /*.7 to show in book*/
         }
         if (show_arrows==1){
           float thisangle;
            if (selflink==0) thisangle=linkangle-(float)PI;
            else thisangle=linkangle;
            /*center the arrow*/
            if (scale_links==1){ 
               poly[0]+=(xend*signx)/2;
               poly[1]+=(yend*signy)/2;
               poly[2]+=(xend*signx)/2;
               poly[3]+=(yend*signy)/2;
            }
	    /*reverse the angle to make arrows point the right way*/
            /*not for self link*/
	        arrow[0]=poly[0]+(poly[2]-poly[0])*.6;
                arrow[1]=poly[1]+(poly[3]-poly[1])*.6;
                arrow[2]=(cos(-(thisangle+PI/10))*w_cyclerad*arrow_size)+arrow[0];
                arrow[3]=(sin(-(thisangle+PI/10))*w_cyclerad*arrow_size)+arrow[1];
                arrow[4]=(cos(-(thisangle-PI/10))*w_cyclerad*arrow_size)+arrow[0];
                arrow[5]=(sin(-(thisangle-PI/10))*w_cyclerad*arrow_size)+arrow[1];
                /*draw the filled arrow*/
	        if (scale_links==0){ 
                   setfillstyle(SOLID_FILL,thiscolor);
                   if (graph_postscript) color_ps(graph_postscript-1,thiscolor);
                } 
                else{ 
                   setfillstyle(SOLID_FILL,white);
                   if (graph_postscript) color_ps(graph_postscript-1,white);
                }         
                fillpoly(3,arrow);
                if (graph_postscript) arrow_block_ps(arrow);         
	 }             
         setcolor(tempcolor);
         if (graph_postscript) color_ps(graph_postscript-1,tempcolor);
}


/****** drag/manipulate graph with mouse , Aug 2000 ********** *******/
void manipulate_jump_graph(signed int wox, signed int woy,
                           signed int X1,signed int Y1,
	                   signed int X2,signed int Y2,
                           float c_scale,float *m_node_size,
                           float rot_angle){

 unsigned long *node_outputs,*node_inputs;
 unsigned int most_recent_node,prev_node,basin_locked_onto=0,
            zw,zz,thiszz;
 signed int promptx,prompty,
             oldx,oldy;
 char test=0,event,components=0,link_step=0,blockset=0,range_type;
 signed int tempx,tempy,bytesT;
 unsigned int wcount,refresh;

    if (no_links==0){
       if ((node_outputs=(unsigned long*)calloc(node_no+1,
                  sizeof(unsigned long)))==NULL) outof_memory("node_outputs");    
    
       if ((node_inputs=(unsigned long*)calloc(node_no+1,
                   sizeof(unsigned long)))==NULL) outof_memory("node_inputs");
    }

    bytesT=(signed int)ceil((double)node_no/8);
    if ((linknet=(char*)calloc(bytesT,sizeof(char)))
				    ==NULL) outof_memory("linknet");

    tempx=getx;
    tempy=gety;
 
    refresh=10000;  
    
#  if __WATCOM__
      refresh=5000;
      if (mouse_found) mouse_call(1,0,0,0); /*display mouse*/
#  elif __UNIX__
       mouse_found=1;          
#  endif

   get_prompt_box(10,3);
  
   components='s';
   most_recent_node=prev_node=1;
   /*if (f_or_n=='n')
       prompt_and_component(components,link_step,most_recent_node-1);
       else  prompt_and_component(components,link_step,most_recent_node);*/
   prompt_and_component(components,link_step,blockset,
                                   most_recent_node,prev_node);

   promptx=getx;
   prompty=gety;

   button_not_rel=button=0;
  
   wcount=0;

   while (1){
      moveto(promptx,prompty); /*position of flashing cursor in prompt box*/
#     if __UNIX__
         /*while(event_ready()==0);*/ /*255 to suppress flashing cursor*/
         flashing_cursor(0,0,0,0,0,-1,CWidth,1+CHeight/2,255);
#     elif __WATCOM__
         flashing_cursor(0,CHeight-vgay(3),CWidth,CHeight,green);
#     endif
     
      if (mouse_found){
	 if (button=='l'|| button=='r'){
	  startmouse:
             wcount+=1;
             if (wcount==Maxu) wcount=1;
	     button_not_rel=0;
              
	    if (button=='l'){
	        if (basin_locked_onto){
		   if (wcount%refresh==0){
		     /*fprintf(stderr,"\n wcount=%lu",count)*/;
		     /*if (button=='l' || button=='r'){
                           //shown in the prompt so not necessary
                           if (blockset==1){
                              if (f_or_n=='n') sprintf(textb[0],
                                   "node %d, %dd Block",zw-1,range_type);
                               else sprintf(textb[0],
                                   "node %d, %dd Block",zw,range_type);
                               inset_prompt_box(0,17,textb[0],'d');
                           }    
                           else{
                              if (f_or_n=='n') sprintf(textb[0],"node %d",zw-1);
                              else sprintf(textb[0],"node %d",zw); 
                              inset_prompt_box(0,9,textb[0],'d');
                           }                                
		      }*/
                      
                      basin_pos[basin_locked_onto][0]=mouse_x;
                      basin_pos[basin_locked_onto][1]=mouse_y;
                      
                      if (components!='s' || blockset==1){
                         xoffset=mouse_x-oldx;
                         yoffset=mouse_y-oldy;                         
                         for (zz=node_no;zz>=1;zz--){
                            thiszz=zz;
			    if (zz==basin_locked_onto ||
				((components!='a')
                                      && get_bit(linknet,thiszz)==0)) 
                                           continue;
			    basin_pos[zz][0]+=xoffset;
                            basin_pos[zz][1]+=yoffset;
                         }
		      } 
                      oldx=basin_pos[basin_locked_onto][0];
                      oldy=basin_pos[basin_locked_onto][1];
                      
                      if (quick_move==0){                     
                         draw_box(-1,X1,Y1,X2,Y2,1,0);/*clear box*/
                         draw_jump_graph(c_scale,m_node_size,wox,woy,X1,Y1,X2,Y2);
                      }
                      else{ 
			 moveto(oldx,oldy);
                         jump_graph_vertex(basin_locked_onto,
			    m_node_size[basin_locked_onto],
                            c_scale,basin_color[basin_locked_onto],
                                  X1,Y1,X2,Y2);
                      }
		   }
		}              
	       /*check left mouse button released*/
#              if __WATCOM__   
	          mouse_call(6,0,0,0);
                  if (globbx==0) button_not_rel='l';
       
#              elif __UNIX__
                  event=XCheckWindowEvent(D,W,ButtonReleaseMask,&Event);
                  /*left button not released*/
                  if (event==0 || Event.xbutton.button!=1) button_not_rel='l'; 
                  /*else fprintf(stderr," event=%d Event==%d notrel=%d\n",
                      event,Event.xbutton.button,button_not_rel);*/   
#              endif
                  /*left button released*/
		  else if (basin_locked_onto){
                     /*lock-on, and left button was released*/
		     basin_pos[basin_locked_onto][0]=mouse_x;
                     basin_pos[basin_locked_onto][1]=mouse_y;
                     if (components!='s' || blockset==1){
                         xoffset=mouse_x-oldx;
                         yoffset=mouse_y-oldy;
                         for (zz=node_no;zz>=1;zz--){
                            if (zz==basin_locked_onto ||
				(components!='a' && 
                                      get_bit(linknet,thiszz)==0)) 
                                           continue;
			    basin_pos[zz][0]+=xoffset;
                            basin_pos[zz][1]+=yoffset;
                         }
		      }

                     draw_box(-1,X1,Y1,X2,Y2,1,0);/*clear box*/
                     draw_jump_graph(c_scale,m_node_size,wox,woy,X1,Y1,X2,Y2);
                     most_recent_node=basin_locked_onto;
                     basin_locked_onto=0; /*unlock*/
		  }
	    }
            
	    if (button=='r'){                                	       
	       /*check right mouse button released*/
#              if __WATCOM__   
	          mouse_call(6,1,0,0);
                  if (globbx==0) button_not_rel='r';
#              elif __UNIX__
                   event=XCheckWindowEvent(D,W,ButtonReleaseMask,&Event);
                   if (event==0 || Event.xbutton.button!=3) button_not_rel='r';
                   /*else fprintf(stderr," event=%d,Event=%d notrel=%d\n",
                           event,Event.xbutton.button,button_not_rel);*/
#              endif	      
	    }
	    
	    if (button_not_rel){
#              if __WATCOM__   		  
                  mouse_call(3,0,0,0); /*mouse position*/
                  mouse_x=globcx;
		  mouse_y=globdx;
                  /* show co-ordinates*/
		  /*sprintf(textb[0],"x=%d y=%d",mouse_x,mouse_y);
			   draw_hint_box(0,0,textb[0],vgax(500));*/   
#              elif __UNIX__
                 if (XCheckWindowEvent(D,W,KeyPressMask|PointerMotionMask,&Event)){
                      if (Event.type == MotionNotify ||
                                    Event.type == ButtonPress){
			 if (Event.type == MotionNotify){
                            mouse_x=Event.xmotion.x;
                            mouse_y=Event.xmotion.y;
                         }
                         if (Event.type == ButtonPress){
                            mouse_x=Event.xbutton.x;
                            mouse_y=Event.xbutton.x;
                         }
                         if (mouse_x>=X1 && mouse_y>=Y1
                               && mouse_x<=X2 && mouse_y<=Y2){
                             if (inside_rect==0){
                                XDefineCursor(D,W,leftCursor);
	                        inside_rect=1;
	                     }
                         }
                         else if (inside_rect==1){
                            XDefineCursor(D,W,rightCursor);
                            inside_rect=0;
                         }
                        /* show co-ordinates*/
			/*sprintf(textb[0],"x=%d y=%d",mouse_x,mouse_y);
			   draw_hint_box(0,0,textb[0],vgax(500));*/       
                     }  
                 }
#              endif
		 if (basin_locked_onto==0){
                    for (zw=node_no;zw>=1;zw--){
		       if (mouse_x>basin_pos[zw][0]-c_scale && 
                          mouse_x<basin_pos[zw][0]+c_scale &&
                          mouse_y>basin_pos[zw][1]-c_scale && 
			  mouse_y<basin_pos[zw][1]+c_scale){
                            prev_node=most_recent_node;
			    basin_locked_onto=most_recent_node=zw;
                            moveto(basin_pos[zw][0],basin_pos[zw][1]);

                            oldx=basin_pos[zw][0];
                            oldy=basin_pos[zw][1];

                            wcount=refresh-1;
                            prompt_and_component(components,blockset,link_step,
			                     most_recent_node,prev_node);
                            break;
	                }
		    }
	         }	 
	         goto startmouse;
	    }
	    continue;
	 }
      }
      test=0;
      test=keycheck(0);
      if (test=='q') goto endFunc;
      if (no_links==0){
         /*disconnect the node*/
         if (test=='g') disconnect_node('g',node_outputs,node_inputs,
	     components,most_recent_node,prev_node);/*cut links to all gap nodes*/
         if (test=='c') disconnect_node('c',node_outputs,node_inputs, 
	     components,most_recent_node,prev_node);/*cut*/
         else if (test=='r') disconnect_node('r',node_outputs,node_inputs,
	     components,most_recent_node,prev_node);/*restore*/
         else if (test=='n') disconnect_node('n',node_outputs,node_inputs,
	     components,most_recent_node,prev_node);/*restore all*/ 
         else if (test=='C') node_pair_link('C',
	     components,most_recent_node,prev_node);/*cut a to b*/
         else if (test=='A') node_pair_link('A',
	     components,most_recent_node,prev_node);/*add a to b*/  
         else if (test=='R') node_pair_link('R',
	     components,most_recent_node,prev_node);/*restore a to b*/ 
         else if (test=='i') components='i';
         else if (test=='o') components='o';
         else if (test=='e') components='e';
         else if (isdigit(test)){
	    sprintf(textb[0],"%c",test);
            link_step=atoi(textb[0]);/*restrict link_step 1-9*/
         }
         else if (test=='0') link_step=0;/*reset link_step to all*/
      }

      if (test=='a') components='a';
      else if (test=='s') components='s';
      /*tog updating network when dragging a node*/
      else if (test=='d') quick_move=abs(quick_move-1); 
      else if (test=='x' || test=='X'){
        char fragment=0;
	if (components=='i' || components=='o' 
	     || components=='e' || blockset==1) fragment=1;
	rotate_component_graph(fragment,test,rot_angle,mouse_x,mouse_y);
      }
      else if (test=='h' || test=='v'){
        char fragment=0;
	if (components=='i' || components=='o' 
	     || components=='e' || blockset==1) fragment=1;
	flip_component_graph(fragment,test,mouse_x,mouse_y);
      }
      /*if (f_or_n=='n')
         prompt_and_component(components,link_step,most_recent_node-1);
	 else  prompt_and_component(components,link_step,most_recent_node);*/
      if (test=='B' && components=='s'){ 
           range_type=block_or_slice(most_recent_node,prev_node);
           blockset=1;
      }
      if (test){
         if (test!='B' && test!='x' && test!='X'
                   && test!='h' && test!='v') blockset=0;

         prompt_and_component(components,blockset,link_step,
                                    most_recent_node,prev_node);
         draw_box(-1,X1,Y1,X2,Y2,1,0);/*clear box*/
                     draw_jump_graph(c_scale,m_node_size,wox,woy,X1,Y1,X2,Y2);
      }
 }
 endFunc:
   if (mouse_found){
#     if __WATCOM__     /*release mouse for rectangle*/
         mouse_call(7,0,0,getmaxx());
         mouse_call(8,0,0,getmaxy());
#     elif __UNIX__
         XDefineCursor(D,W,rightCursor); /*restore cursor shape*/
#     endif
   }   
   uncover_free_prompt_box(10);

   free(linknet);
   if (no_links==0){
      free(node_outputs);
      free(node_inputs);
   }
   moveto(tempx,tempy);
}

/************** draw drag prompt and check component * *****************/
void prompt_and_component(char components, char blockset, char link_step,
        unsigned int most_recent_node, unsigned int prev_node){
  signed int xbox,bytesT;
  unsigned int m_r_node,p_node;
  
         xbox=draw_prompt_box(4);
         setcolor(lightred);
         
         if (f_or_n=='n'){ /*network indices start from 0*/
	    m_r_node=most_recent_node-1;
            p_node=prev_node-1;
            bytesT=bytesL;
         }
         else{
            m_r_node=most_recent_node;
            p_node=prev_node;
            bytesT=(signed int)ceil((double)node_no/8);
         }
         if (blockset==0){ 
            free(linknet); /*empty the list of linked nodes*/
            if ((linknet=(char*)calloc(bytesT,sizeof(char)))
				    ==NULL) outof_memory("linknet");
         }
         if (link_step==0) sprintf(textb[1],"step=nolimit");
         else if (link_step>0) sprintf(textb[1],"step=%d",link_step);

         if (components=='i')
            sprintf(textb[0],"node %d, inputs, %s: ",
                                           m_r_node,textb[1]);
         else if (components=='o') 
            sprintf(textb[0],"node %d, outputs, %s: ",
                                           m_r_node,textb[1]);
         else if (components=='e')
            sprintf(textb[0],"node %d, either, %s: ",
                                           m_r_node,textb[1]);
         else if (components=='a') sprintf(textb[0],"all nodes: ");
         else if (components=='s')
                            sprintf(textb[0],"node %d, single: ",m_r_node);
         if (blockset==1) 
                     sprintf(textb[0],"node %d, Block %d-%d: ",m_r_node,
                                        startBlock,endBlock);

         outtext(textb[0]); 
         if (components=='i' || components=='o'||  components=='e'){
               graph_in_out_components(
                                 components,link_step,most_recent_node);
         }
         
         setcolor(white);
         outtext("drag - left button, tog drag-d");
         moveto(xbox,gety+CHeight);
         if (no_links==0) outtext(
            "not active?-click right button first, rotate-x/X flip-h/v gap-g");
         else outtext("not active?-click right button first, rotate-x/X flip-h/v");
         moveto(xbox,gety+CHeight);
         if (no_links==0){
            if (components=='i' || components=='o' || components=='e')   
	    sprintf(textb[0],
            "links %d: cut/restore/net-c/r/n, link %d- %d: cut/add/restore-C/A/R",
                            m_r_node,m_r_node,p_node);
            else if (components=='s')
              sprintf(textb[0],"links: restore net-n, block-B");
            else sprintf(textb[0],"links: restore net-n");
	    outtext(textb[0]);
         }
         else{
            sprintf(textb[0],"block-B");
	    outtext(textb[0]);
         } 
         moveto(xbox,gety+CHeight);
         if (no_links==0){
            outtext(
            "step-(1-9) nolimit-0 single-s in/out/either-i/o/e all-a exit-q:");
         }
         else outtext("single-s all-a exit-q:");
}

/************** define a block, or slice in 2d or 3d *****************/
char block_or_slice(unsigned int first,unsigned int last){

             unsigned int *ijh,*ijh_in,*ijh_def_low,*ijh_def_high,
	          temp_last,start,end,zi,
                  first_ii,first_jj,first_hh,
                  last_ii,last_jj,last_hh,
	          low_ii,high_ii,low_jj,high_jj,low_hh,high_hh,
	          this_ii,this_jj,this_hh;
             signed int z,zz,xbox,bytesT;
             char range_type;

                       if (f_or_n=='n') bytesT=bytesL;
                       else bytesT=(signed int)ceil((double)node_no/8);
                       free(linknet); /*empty the list of linked nodes*/
                       if ((linknet=(char*)calloc(bytesT,sizeof(char)))
				    ==NULL) outof_memory("linknet");
             
                       if ((ijh=(unsigned int *)calloc(3,sizeof(unsigned int)))
				   ==NULL) outof_memory("ijh");
                       if ((ijh_in=(unsigned int *)calloc(3,sizeof(unsigned int)))
				   ==NULL) outof_memory("input_ijh");
                       if ((ijh_def_low=(unsigned int *)calloc(3,sizeof(unsigned int)))
				   ==NULL) outof_memory("input_def_low");
                       if ((ijh_def_high=(unsigned int *)calloc(3,sizeof(unsigned int)))
				   ==NULL) outof_memory("input_def_high");

                       /*make sure first is smaller than last, else swap*/
                       if (first>last){
			  temp_last=last;
                          last=first;
                          first=temp_last;
                       }
                       
                       if (f_or_n=='n'){
                          start=0;
                          end=node_no-1;                          
                          first--;
			  last--;
                          if (threed) range_type=3;
                          else if (twod) range_type=2;
                          else if (oned) range_type=1;
                       }
                       else{ 
			  start=1;
                          end=node_no;
                          range_type=1;
                       }
                       xbox=draw_prompt_box(4);
                       if (f_or_n=='n' && (threed || twod)){
                          if (threed) sprintf(textb[0],
                             "3d network: define block: 1d-1 3d-(def):");
                          else if (twod) sprintf(textb[0],
                             "2d network: define block: 1d-1, 2d-(def):");
                       }
                       else sprintf(textb[0],"define 1d block:");
                       outtext(textb[0]);
                       input_string(1,0);
                       if (is[0]=='q') goto quit_b;
                       if (atoi(is)==1) range_type=1;

                       moveto(xbox,gety+CHeight);

                       if (range_type==1){ 
                          first_ii=first;
                          last_ii=last;
                       }
                       else if (range_type==2){
                          first_ii=first%i2d;
	                  first_jj=first/i2d; 
                          last_ii=last%i2d;
	                  last_jj=last/i2d;                            
                       }                                	
                       else if (range_type==3){
                          first_ii=first%i3d;
	                  first_jj=first%(i3d*j3d)/i3d;
                          first_hh=first/(i3d*j3d);
                          last_ii=last%i3d;
	                  last_jj=last%(i3d*j3d)/i3d;
                          last_hh=last/(i3d*j3d);
		       }                                   

                       if (range_type==1){
                             unsigned int low_v,high_v;
                             low_v=high_v=lineL-1;                       
                             if (f_or_n=='n') sprintf(textb[0], 
                                "%dd block (%d-%d, def %d-%d), ",
                                      range_type,start,end,first_ii,last_ii);
                             else sprintf(textb[0], 
                                "%dd block (%d-%d, def %d-%d), ",
                                      range_type,start,end,first_ii,last_ii);
                             outtext(textb[0]);
                             moveto(xbox,gety+CHeight);
                             outtext("low:");
                             input_string(5,0);
                             if (is[0]=='q') goto quit_b;
                             else if (is[0]!=13 && atoi(is)>=start && 
                               atoi(is)<=end) low_v=atoi(is);
                             else low_v=first_ii;
                             moveto(xbox,gety+CHeight);
                             outtext("high:");
                             input_string(5,0);
                             if (is[0]=='q') goto quit_b;
                             else if (is[0]!=13 && atoi(is)>=start && 
                                atoi(is)<=end && atoi(is)>=low_v) high_v=atoi(is);
                             else high_v=last_ii;
                             /*make sure first co-ord is smaller than last, else swap*/
                                if (low_v>high_v){
			           temp_last=high_v;
                                   high_v=low_v;
                                   low_v=temp_last;
                                }
			     for (zi=low_v;zi<=high_v;zi++){
                                if (f_or_n=='n') set_bit1(linknet,zi+1);
                                else set_bit1(linknet,zi);
                             }           
                       } 
                       
                       else if (range_type==2){                        
                             ijh[0]=i2d;
                             ijh[1]=j2d;
                             ijh_def_low[0]=first_ii;
                             ijh_def_low[1]=first_jj;
                             ijh_def_high[0]=last_ii;
                             ijh_def_high[1]=last_jj;
                             
                             sprintf(textb[0],
	                      "%dd block (this=%d,%d max=%d,%d)",
                                range_type,first_ii,first_jj,i2d-1,j2d-1);
                             outtext(textb[0]);
                             for (zz=1;zz<=2;zz++){ 
                                moveto(xbox,gety+CHeight);    
                                if (zz==1) sprintf(textb[0],
                                      "low corner (def %d,%d)",first_ii,first_jj);
                                else if (zz==2) sprintf(textb[0],
                                      "high corner (def %d,%d)",last_ii,last_jj);
                                outtext(textb[0]);
                                sprintf(textb[0],"%s"," i:");
                                sprintf(textb[1],"%s"," j:");
                             
                                for (z=0;z<=1;z++){
                                   outtext(textb[z]);
                                   input_string(4,0);
                                   if (is[0]=='q') goto quit_b;
                                   else if (is[0]!=13 && atoi(is)>=start && 
                                      atoi(is)<=ijh[z]-1) ijh_in[z]=atoi(is);
                                   else{ 
                                      if (zz==1) ijh_in[z]=ijh_def_low[z];
                                      else if (zz==2) ijh_in[z]=ijh_def_high[z];
                                   }
                                }
			        if (zz==1){
                                   low_ii=ijh_in[0];
                                   low_jj=ijh_in[1];         
                                } 
                                else if (zz==2){
                                   high_ii=ijh_in[0];
                                   high_jj=ijh_in[1];
                                }
			     }
                             /*make sure first co-ord is smaller than last, else swap*/
                             if (low_ii>high_ii){
			           temp_last=high_ii;
                                   high_ii=low_ii;
                                   low_ii=temp_last;
                             }
                             if (low_jj>high_jj){
			           temp_last=high_jj;
                                   high_jj=low_jj;
                                   low_jj=temp_last;
                             }  
                             first=i2d*low_jj + low_ii;
                             last=i2d*high_jj + high_ii;
                             /*fprintf(stderr,"\nfirst=%d last=%d",first,last);*/
                             for (zi=first;zi<=last;zi++){
                                 this_ii=zi%i2d;
                                 this_jj=zi/i2d;
                                 if (this_ii<low_ii || this_ii>high_ii ||
                                    this_jj<low_jj || this_jj>high_jj) continue;
                                 else set_bit1(linknet,zi+1);
                             }
                       } 
                       else if (range_type==3){                          
                             ijh[0]=i3d;
                             ijh[1]=j3d;
                             ijh[2]=h3d;

                             ijh_def_low[0]=first_ii;
                             ijh_def_low[1]=first_jj;
                             ijh_def_low[2]=first_hh;
                             ijh_def_high[0]=last_ii;
                             ijh_def_high[1]=last_jj;
                             ijh_def_high[2]=last_hh;
                                                 
                             sprintf(textb[0],
	                      "%dd block (this=%d,%d,%d max=%d,%d,%d)",
                                range_type,first_ii,first_jj,first_hh,
                                                   i3d-1,j3d-1,h3d-1);
                             outtext(textb[0]);
                             for (zz=1;zz<=2;zz++){ 
                                moveto(xbox,gety+CHeight);    
                                if (zz==1) sprintf(textb[0],
                                     "low corner (def %d,%d,%d)",
                                              first_ii,first_jj,first_hh);
                                else if (zz==2) sprintf(textb[0],
                                   "high corner (def %d,%d,%d)",
                                       last_ii,last_jj,last_hh);
                                outtext(textb[0]);
                                sprintf(textb[0],"%s"," i:");
                                sprintf(textb[1],"%s"," j:");
                                sprintf(textb[2],"%s"," h:");
                             
                                for (z=0;z<=2;z++){
                                   outtext(textb[z]);
                                   input_string(4,0);
                                   if (is[0]=='q') goto quit_b;
                                   else if (is[0]!=13 && atoi(is)>=0 && 
                                      atoi(is)<=ijh[z]-1) ijh_in[z]=atoi(is);
                                   else{
                                      if (zz==1) ijh_in[z]=ijh_def_low[z];
                                      else if (zz==2) ijh_in[z]=ijh_def_high[z];
                                   }
                                }
			        if (zz==1){
                                   low_ii=ijh_in[0];
                                   low_jj=ijh_in[1];
                                   low_hh=ijh_in[2];
                                } 
                                else if (zz==2){
                                   high_ii=ijh_in[0];
                                   high_jj=ijh_in[1];
                                   high_hh=ijh_in[2];
                                }  
			     }
                             /*make sure first co-ord is smaller than last, else swap*/
                             fprintf(stderr,
                                 "\n1:low_ii=%d low_jj=%d low_hh=%d high_ii=%d high_jj=%d high_hh=%d",
                                              low_ii,low_jj,low_hh,high_ii,high_jj,high_hh);
                             /*make sure first co-ord is smaller than last, else swap*/
                             if (low_ii>high_ii){
			           temp_last=high_ii;
                                   high_ii=low_ii;
                                   low_ii=temp_last;
                             }
                             if (low_jj>high_jj){
			           temp_last=high_jj;
                                   high_jj=low_jj;
                                   low_jj=temp_last;
                             }
                             if (low_hh>high_hh){
			           temp_last=high_hh;
                                   high_hh=low_hh;
                                   low_hh=temp_last;
                             }
                             first=(i3d*j3d*low_hh) + (i3d*low_jj) + low_ii;
                             last=(i3d*j3d*high_hh) + (i3d*high_jj) + high_ii;
                             for (zi=first;zi<=last;zi++){
                                 this_ii=zi%i3d;
                                 this_jj=zi%(i3d*j3d)/i3d;
                                 this_hh=zi/(i3d*j3d);
                                 if (this_ii<low_ii || this_ii>high_ii ||
                                    this_jj<low_jj || this_jj>high_jj ||
                                       this_hh<low_hh || this_hh>high_hh) continue;
                                 else set_bit1(linknet,zi+1);
                             }
                                    
                       } 
                       /*fprintf(stderr,"\n linknet=");
                       for (zi=start;zi<=end;zi++)
		       fprintf(stderr,"%d",get_bit(linknet,zi));*/

                       startBlock=endBlock=0;
                       for (zi=start;zi<=end;zi++){
			  if (get_bit(linknet,zi)==1){ 
                              startBlock=zi;
                              break;
                          }
                       }
                       for (zi=end;zi>=startBlock;zi--){
			  if (get_bit(linknet,zi)==1){ 
                              endBlock=zi;
                              break;
                          }
                       }
                       if (f_or_n=='n'){
			 startBlock-=1;
                         endBlock-=1;
                       }
                      quit_b:
                        free(ijh);
                        free(ijh_in);
                        free(ijh_def_low); 
                        free(ijh_def_high); 
                        return(range_type);                    
}

/************** disconnect a node  ******************/
void disconnect_node(char what,
	    unsigned long *node_outputs, unsigned long *node_inputs,
            char components, unsigned int poss, unsigned int prev_node){
  unsigned int zu,z;

      if (what=='n'){
         for (zu=node_no;zu>=1;zu--)
	        memmove(basinjumps[zu],b_j[zu],
                            (node_no+1)*sizeof(unsigned long));
      } 
      
      if (what=='g'){ /*gap node: can't transmit info*/
	              /*if no outside outputs, or no outside inputs*/
            /*recalculate node_outputs, node_inputs*/
	    for (zu=1;zu<=node_no;zu++){
	       node_outputs[zu]=0;
               node_inputs[zu]=0;
            }            

            for (zu=1;zu<=node_no;zu++){
               for (z=1;z<=node_no;z++){
		  if (z!=zu ){ /*don't count self outputs*/		     
                     node_outputs[zu]+=basinjumps[zu][z];
                     node_inputs[zu]+=basinjumps[z][zu];
                  }
	       }
            }
            /*fprintf(stderr,"\n before");
            for (z=1;z<=node_no;z++){
               if (f_or_n=='n')
                 fprintf(stderr,"\n %d: out=%d in=%d",z-1,node_outputs[z],node_inputs[z]);
               else  fprintf(stderr,"\n %d: out=%d in=%d",z,node_outputs[z],node_inputs[z]);
	    }*/
            for (zu=node_no;zu>=1;zu--){ /*output links * input links*/
               if (node_outputs[zu]==0 || node_inputs[zu]==0){
                  for (z=node_no;z>=1;z--){
                      basinjumps[zu][z]=0; /*cut outputs*/
                      basinjumps[z][zu]=0; /*cut inputs*/
                  }
	       }
            }            
      } 
    
      if (components=='i' || components=='e'){ 
	 for (zu=node_no;zu>=1;zu--){
	     if (what=='c') basinjumps[zu][poss]=0; /*cut inputs*/ 
             else if (what=='r') /*restore inputs*/ 
	       basinjumps[zu][poss]=b_j[zu][poss];            
         }         
      }
      if (components=='o' || components=='e'){ 
	 for (zu=node_no;zu>=1;zu--){
	     if (what=='c') basinjumps[poss][zu]=0; /*cut outputs*/
             else if (what=='r') /*restore outputs*/
                basinjumps[poss][zu]=b_j[poss][zu];
         }
      }
}

/*************** cut links between pairs of nodes  *******************/
void node_pair_link(char what,
            char components, unsigned int poss, unsigned int prev_node){
  
      if (components=='i' || components=='e'){ 
	     if (what=='C') basinjumps[prev_node][poss]=0; /*cut input*/ 
             else if (what=='A'){ /*add input*/ 
	       if (basinjumps[prev_node][poss]==0)
		     basinjumps[prev_node][poss]=1;
             }
             else if (what=='R') /*restore input*/ 
	       basinjumps[prev_node][poss]=
                        b_j[prev_node][poss];
      
      }
      if (components=='o' || components=='e'){  
	     if (what=='C') basinjumps[poss][prev_node]=0; /*cut output*/
             else if (what=='A'){ /*add input*/ 
	       if (basinjumps[poss][prev_node]==0)
		     basinjumps[poss][prev_node]=1;
             }
             else if (what=='R') /*restore outputs*/
                basinjumps[poss][prev_node]=
                            b_j[poss][prev_node];
         
      }
}

/************** show all input or output links to cell ******************/
void graph_in_out_components(char components, char link_step,
                                          unsigned int poss){

  char *this_sweep,*genedone,linkfound;
  signed int t_step=1,bytesT;
  unsigned int startposs,
                 zu,zu1,tot_so_far=0,tot_before=0;           

     bytesT=(signed int)ceil((double)node_no/8);
     if ((genedone=(char*)calloc(bytesT,sizeof(char)))
				==NULL) outof_memory("genedone");
     if ((this_sweep=(char*)calloc(bytesT,sizeof(char)))
				==NULL) outof_memory("this_sweep");
             
      /*mark all input genes to startpos*/
      startposs=poss;
      /*fprintf(stderr,"\njump: pos=%d",startposs);*/    
      if (components=='i' || components=='e'){ /*show inputs*/ 
         for (zu=node_no;zu>=1;zu--){
            if (basinjumps[zu][startposs]>0)
               set_bit1(linknet,zu);
	       tot_so_far+=1;
         }
	 
      }
      if (components=='o' || components=='e'){ /*show outputs*/
	 for (zu=node_no;zu>=1;zu--){
            if (basinjumps[startposs][zu]>0)
               set_bit1(linknet,zu);
	       tot_so_far+=1;
         }
      }
      t_step=2;
      if (link_step==1) goto endFunc;

      while(1){          
         linkfound=0;        
         for (zu=node_no;zu>=1;zu--){                           
            if (get_bit(genedone,zu)==0 && get_bit(linknet,zu)==1){
	       linkfound=1;
	       set_bit1(genedone,zu);
               if (components=='i' || components=='e'){  /*inputs*/
                  for (zu1=node_no;zu1>=1;zu1--){
                     if (basinjumps[zu1][zu]>0)
                           set_bit1(this_sweep,zu1);	    
                  }   
               }                

               if (components=='o' || components=='e'){  /*outputs*/             
                    for (zu1=node_no;zu1>=1;zu1--){
                        if (basinjumps[zu][zu1]>0)
                              set_bit1(this_sweep,zu1);	    
                    }   
               }
	    }
            else continue;
         }

         for (zu=node_no;zu>=1;zu--){
            if (get_bit(this_sweep,zu)==1){ 
               set_bit1(linknet,zu);
               set_bit(this_sweep,zu,0);
            }
         }
         tot_before=tot_so_far;
         tot_so_far=0;
         for (zu=node_no;zu>=1;zu--){                   
            if (get_bit(linknet,zu)==1) tot_so_far+=1;
         }
         if (link_step>0 && t_step==link_step) goto endFunc;
         if (linkfound==0 || tot_before*2==tot_so_far) goto endFunc;
         /*if (linkfound==1 && tot_before<tot_so_far) t_step+=1;*/
         t_step+=1;
      }
    endFunc:
      /*fprintf(stderr,"\n step=%d, %d/%d=%.1f%%\n",
            t_step,tot_so_far,
	    node_no,100.0*((float)tot_so_far/node_no));
      for (zu=node_no;zu>=1;zu--){
	fprintf(stderr," %d",zu);                   
        if (get_bit(linknet,zu)==1) fprintf(stderr,"=1");
        if (get_bit(linknet,zu)==1) fprintf(stderr,"1");
        else fprintf(stderr,"-");        
      } */            
      poss=startposs;      
      free(genedone);
      free(this_sweep);                    
}


/****** compute link angle and absolute angle (<90) from 2 points *******/ 
void compute_link_angle(signed int x1, signed int y1,
                        signed int x2, signed int y2){ 
  float ratio;
       
      /* rads to degs *57.2958, deg to rads *0.0174533 */  
      if (x1==x2){
         if (y1>y2) linkangle=(float)(PI/2);
         else if (y1<y2){
            linkangle=(float)(3*PI/2);
         }
         abs_angle=(float)(PI/2);
      }
      else if (y1==y2){
         if (x1>x2) linkangle=(float)PI;
         else if (x1<x2) linkangle=(float)(2*PI);
         abs_angle=0; 
      }
      
      else {
	 ratio=(float)fabs((float)(y1-y2)/(float)(x1-x2));       
	 abs_angle = (float) atan (ratio);
         /*

  PI/2-PI    y2<y1  |   y2<y1    0-PI/2
  PI-h       x2<x1  |   x2>x1 ...ok
           ---------|--------
  PI-3*PI/2  y2>y1  |   y2>y1
  PI+h       x2<x1  |   x2>x1  3*PI/2 - 2*PI-h  */

         if      (x1<=x2 && y1>y2) linkangle=abs_angle; 	 
         else if (x1>x2 && y1>=y2) linkangle=(float)PI-abs_angle;
         else if (x1>=x2 && y1<y2) linkangle=(float)PI+abs_angle;        
         else if (x1<x2 && y1<=y2) linkangle=(float)(2*PI)-abs_angle; 
      }     
}  

/*************************************************************************/
